/*@***************************************************************************
@
@       PROJECT         : XB: Sorter Control
@
@       FILE            : CELL.H
@
@       VERSION         : XB 5.0
@
@       TYPE            : INCLUDE
@
@
@       DESCRIPTION     : Structure for Cell Data Table
@
@       COMPILER        : ic386
@
@       OPERATING SYSTEM: iRMX for Windows
@
@       NOTE            : 20/02/95 status bit used for the cts_status field
@													by Carrier Test Station added
@                         19/09/1995 bit and mask value used to set the cell,
@                         decoder and converter errors added
@                         17/01/96 set enable and chute
@                         16/11/96 Macro modification for DXB
@													11/07/97 New "RECIRC_CONSEC_OFFL" and
@													and "RECIRC_MISSION_LOST" recirculation reason
@                         13/07/00 Extra bit CELL_ITEM_FOUND for the cell status
@                         added
@                         2002 Add QUAD definitions for cells
@
@***************************************************************************/

/* table declaring */
#ifndef TABSCCELL
#define TABSCCELL

#define CELL ps_cell_ele
#define CELL_HEADER gps_cell_header

#define FIRST_LOWER_LOOP_CELL	1
#define LAST_LOWER_LOOP_CELL	999
#define FIRST_UPPER_LOOP_CELL	1001
#define LAST_UPPER_LOOP_CELL	1999


/*@.....................................................................
@
@ Begin of CELL table definition
@
@.....................................................................*/

/*.........................................................................
@
@ CELL defines
@
@.......................................................................*/
#define FILE_CELL "CELL.DAT"  /* File for saving the cell data */
/*.........................................................................
@
@ Cell macro
@
@.......................................................................*/


/*@.....................................................................
@ LOWER_LOOP_CELL -> it tests if the cell is in the lower loop
@.....................................................................*/
#define LOWER_LOOP_CELL \
		(CELL->id<=LAST_LOWER_LOOP_CELL)

/*@.....................................................................
@ UPPER_LOOP_CELL -> it tests if the cell is in the UPPER loop
@.....................................................................*/
#define UPPER_LOOP_CELL \
		(CELL->id >= FIRST_UPPER_LOOP_CELL)

/*@.....................................................................
@ GET_CELL_OPPOSITE -> it gets the opposite cell pointer (for QUAD only)
@.....................................................................*/
#define GET_CELL_OPPOSITE \
	if (QUAD_SORTER)\
		{ \
		if (LOWER_LOOP_CELL)\
			CELL=CELL+gn_cell/2;\
		else\
			CELL=CELL-gn_cell/2;\
		} 

/*@.....................................................................
@ GET_CELL_PREVIOUS -> it assignes the pointer to the CELL table for the previous cell
@.....................................................................*/
#define GET_CELL_PREVIOUS \
	if (SINGLE_DECK_SORTER)\
		{ \
		if (CELL->id==1) \
			CELL = gps_cell+gn_cell-1; \
		else  \
			CELL--; \
		} \
	else \
		{ \
		if (CELL->id==1) \
			CELL=gps_cell+gn_cell/2-1; \
		else if (CELL->id==FIRST_UPPER_LOOP_CELL) \
			CELL=gps_cell+gn_cell-1; \
		else \
			CELL--; \
		}
 

/*@.....................................................................
@ GET_CELL_NEXT -> it assignes the pointer to the CELL table for the next cell
@.....................................................................*/
#define GET_CELL_NEXT \
	if (SINGLE_DECK_SORTER)\
		{ \
		if (CELL->id == gn_cell)   \
			CELL = gps_cell;  \
		else \
			CELL++; \
		} \
	else \
		{ \
		if (CELL->id==(gps_cell+gn_cell-1)->id) \
			CELL=gps_cell+gn_cell/2; \
		else if (CELL->id==(gps_cell+gn_cell/2-1)->id) \
			CELL=gps_cell;  \
		else \
			CELL++; \
		}
 

/*@.....................................................................
@ GET_CELL -> it gets the cell pointer for the COORD position (in ep)
@.....................................................................*/
#define GET_CELL(COORD); \
	CELL=(S_cell *)get_cell_adr(COORD);

/*@.....................................................................
@ GET_CELL_FROM_ID -> it gets the cell pointer for the cell number
@.....................................................................*/
#define GET_CELL_FROM_ID(N_CELL);  \
	if (SINGLE_DECK_SORTER)\
		{\
		CELL=gps_cell+N_CELL-1;\
		}\
	else\
		{\
		if (N_CELL<FIRST_UPPER_LOOP_CELL)\
			CELL=gps_cell+N_CELL-1;\
		else\
			CELL=gps_cell+gn_cell/2+N_CELL-FIRST_UPPER_LOOP_CELL;\
		}

/*@.....................................................................
@
@ TEST_CELL_ID_AND_GET_CELL -> it gets the cell pointer for the cell number
@
@.....................................................................*/
#define TEST_CELL_ID_AND_GET_CELL(N_CELL)\
	if (SINGLE_DECK_SORTER)\
		{\
		if (N_CELL<1 || N_CELL>gn_cell)\
			CELL=0;\
		else\
			CELL=gps_cell+N_CELL-1;\
		}\
	else\
		{\
		if (N_CELL<1\
		|| N_CELL>(LAST_LOWER_LOOP_CELL+gn_cell/2+1)\
		|| (N_CELL>gn_cell/2 && N_CELL<FIRST_UPPER_LOOP_CELL))\
			CELL=0;\
		else if (N_CELL<FIRST_UPPER_LOOP_CELL)\
			CELL=gps_cell+N_CELL-1;\
		else\
			CELL=gps_cell+gn_cell/2+N_CELL-FIRST_UPPER_LOOP_CELL;\
		}


/*@.....................................................................
@ SINGLE_CELL -> it tests if the cell is single
@.....................................................................*/
#define SINGLE_CELL ( !TESTBIT(CELL->status,CELL_DOUBLE) )

/*@.....................................................................
@ FIRST_OF_DOUBLE_CELL -> it tests if the cell is the first of a double cell
@.....................................................................*/
#define FIRST_OF_DOUBLE_CELL \
	( TESTBIT(CELL->status,CELL_DOUBLE) && \
		TESTBIT(CELL->status,CELL_FIRST_OF_DOUBLE) )

/*@.....................................................................
@ SECOND_OF_DOUBLE_CELL -> it tests if the cell is the second of a double cell
@.....................................................................*/
#define SECOND_OF_DOUBLE_CELL \
	( TESTBIT(CELL->status,CELL_DOUBLE) &&  \
		!TESTBIT(CELL->status,CELL_FIRST_OF_DOUBLE) )

/*@.....................................................................
@ DOUBLE_CROSS_CELL -> it tests if the cell is a double cross cell
@.....................................................................*/
#define DOUBLE_CROSS_CELL \
	( TESTBIT(CELL->status,CELL_DOUBLE_CROSS) )
/*.........................................................................
@
@ End of cell macro
@
@.......................................................................*/

/*@.....................................................................
@
@ Define for the cell status field
@
@.....................................................................*/

#define CELL_BUSY                     0
#define CELL_LOADABLE                 1
#define CELL_TO_VERIFY                2
#define CELL_DOUBLE                   3
#define CELL_FIRST_OF_DOUBLE          4
#define CELL_REQ_OFFLOADING           5
#define CELL_REQ_REJECT               6
#define CELL_FAILURE                  7

/* status bit used only by Load Stations
(LOADMGM task) */
#define CELL_RESERVED                 8
#define CELL_IN_LOADING               9
#define CELL_REQ_SCANNER              10

/* status bit set by set_offloading_cnt routine
	and used by the OFFLVER,LOADVER and OFFLMGM tasks */
#define CELL_OFFL_RESERVED            11

/* status bit used only for cell movement without data */
#define CELL_FALSE_MOVE               12

/* status bit used only by the OFFLVER task */
#define CELL_REQ_OFFL_BEFORE_OFFLVER  13

/* status bit used by OFFLMGM and OFFLVER task */
#define CELL_RECIRCULATING            14

/* status bit used by Virtual Scanner
and Offloading Verification Station  */
#define CELL_WAIT_REDEST              15

/* status bit used only by Recentering Station
(RECMGM task)*/
#define CELL_BADPOS_RECENT            16
#define CELL_NOCHECK_RECENT           17

/* status bit used only for test a cell at the reject chute
(set by OFFLVER task and reset by OFFLMGM task */
#define CELL_FALSE_REJECT             18

/* bit for parcel result already sent */
#define CELL_PARCRES_SENT             19

/* bit for delayed tracking OK */
#define CELL_TRACKING_OK              20

/* bit for item seen by the footprint station */
#define CELL_ITEM_FOUND               21

/* New status bit for quad sorter cells */

/* status bit set by sidemgm to test movement
from side to side of items */
#define CELL_MOVED_TO_VERIFY          30
 
/* status bit for item cross loaded */
#define CELL_DOUBLE_CROSS		          31

/*@.....................................................................
@
@ Define for the Cell Command status field
@
@.....................................................................*/

#define CELL_OFFL_REQUEST           0
#define CELL_OFFL_DELETING_REQUEST  1
#define CELL_OFFL_DONE              2
#define CELL_OFFL_DELETED           3

/* Status bit used for the cts_status field by Carrier Test Station */
#define CELL_TEST_OK        0
#define CELL_RUN_TEST       1
#define CELL_UNDER_TEST     2

/* status bit set/reset by the TESTMGM and used by the LOADVER task */
#define CELL_TEST_FAILURE   3

/* bit and mask value used to set the cell, decoder and converter errors */
#define CELL_ERROR            0
#define DECODER_ERROR         1
#define CONVERTER_ERROR       	  2
#define SET_CELL_ERROR        (1<<CELL_ERROR)
#define SET_DECODER_ERROR     (1<<DECODER_ERROR)
#define SET_CONVERTER_ERROR   (1<<CONVERTER_ERROR)

/*@.....................................................................
@
@ Recirculation reasons: index for the "recirc_cnt" field
@
@.....................................................................*/
enum recirculation_reason
	{
	RECIRC_CHUTE_DISABLED,
	RECIRC_CHUTE_FULL,
	RECIRC_CONSEC_OFFL,
	RECIRC_MISSION_LOST,
	RECIRC_NO_READ,
	RECIRC_OUT_OF_BATCH,
	RECIRC_NO_REDEST,
	RECIRC_GENERAL,
	MAX_RECIRC_REASON
	};

/*@.....................................................................
@
@ Reject reasons defines
@
@.....................................................................*/
enum reject_reason
	{
	REJECT_CHUTE_FULL,          /* 1    */
	REJECT_CHUTE_DISABLED,      /* 2    */
	REJECT_NO_READ,             /* 4    */
	REJECT_OUT_OF_BATCH,        /* 8    */
	REJECT_PARCEL_NO_DATA,      /* 0x10		*/
	REJECT_PARCEL_LOST,         /* 0x20		*/
	REJECT_NO_REDEST,           /* 0x40		*/
	REJECT_BAD_LOADED,          /* 0x80		*/
	REJECT_TRACK_ERR,           /* 0x100	*/
	REJECT_MULT_LABEL,          /* 0x200	*/
	REJECT_CHUTE_NOT_EXIST,     /* 0x400	*/
	REJECT_OTHER,               /* 0x800	*/
	REJECT_BADPOS_RECENT,       /* 0x1000 */
	REJECT_CELL_FAILURE,        /* 0x2000 */
	MAX_REJECT_REASON
	};
/*@.....................................................................
@
@ Begin of CELL table definition
@
@.....................................................................*/


/*@.....................................................................
@
@ CELL local structures
@
@.....................................................................*/

/*@.....................................................................
@ Local structure for Cell Data Table
@.....................................................................*/
/*@.....................................................................
@
@ Cell configuration data structure
@
@.....................................................................*/

typedef struct
	{
	USHORT  width;              /* cell width in mm */
	USHORT  pitch;              /* cell pitch in mm */
	USHORT  top_shield_length;  /* cell pitch in mm */
	USHORT  brush_distance;     /* distance in mm between the begin of the cell
																 belt and the brushes */
	}S_cfg_cell;

/*@.....................................................................
@
@ Cell Command Activation request structure
@
@.....................................................................*/

typedef struct
	{
	short   cnt;           /* Encoder Pulse counter for Cell movement */
	USHORT  enable;        /* Enable for Cell Command Activation */
	USHORT  chute;         /* 1 = countdown of the "cnt" field
														for Cell Command Activation enabled */
	short   offset;         /* distance in mm from the start
														 of the belt to the head of the parcel
														 (negative if the parcel is on the top-shield) */
	USHORT  relative_ep;    /* relative EP when the offset is updated */
	USHORT  absolute_ep;    /* absolute EP when the offset is updated */
	} S_move;

/*@.....................................................................
@
@ Recentering information structure:
@ used by cross function if it has not valid information
@
@.....................................................................*/
typedef struct
	{
	USHORT  no_move;            /* 1 = no action */
	long  cross_displacement;   /* displacement required
																 >0 CCW direction
																 <0 CW direction */
	long  norm_displacement;    /* displacement required
																 >0 CW direction
																 <0 CCW direction */
	}S_recent_req;

/*@.....................................................................
@
@ working data for the cell
@
@.....................................................................*/

typedef struct
	{
	USHORT  test_direction;         /* CW_DIRECTION = clock wise direction
																	   CCW_DIRECTION = counter clock wise direction */
	USHORT  n_test_error;           /* number of test station error */
	USHORT  recirc_reason;          /* recirculation reasons */
	USHORT  n_error;                /* counter for cell error */
	USHORT  recirc_cnt[MAX_RECIRC_REASON];  /* counter for recirculation reason */
	USHORT  n_isc;                 	/* ISC that reserved the cell */
	S_id    s_id_decoder;          	/* relevant decoder */
	S_id    s_id_celcmd;           	/* relevant converter (last converter used
																		 by the CCMGM task for the cell) */
	S_id    s_id_offlver;          	/* relevant Offloading Verification Station */
	USHORT  n_cons_error_cts;     	/* number of consecutive errors under the CTS */
	USHORT  cnt_parc_worked;				/* number of parcel worked without test */
	USHORT  cnt_xb_laps;		       	/* number of xb laps without test */
	USHORT  test_type;	           	/* type of test made at least once:
																		 bit 0: CW_DIRECTION
																		 bit 1: CCW_DIRECTION */
	USHORT  cnt_cts_failure;	     	/* number of consecutive failure under CTS */
	short   offl_offset;   					/* generic offset to be used during the offloading */
	short   offl_offset_scweight;		/* offset to be used during the offloading
																		 set by WEIGHT */
	short   offl_offset_scdim;    	/* offset to be used during the offloading
																		 set by DIM */
	USHORT  n_chute_last_off;				/* last chute number where was offloaded the parcel */
	USHORT  n_lap_last_off;					/* last lap that the cell was offloaded */
	} S_wk_cell;


/*@.....................................................................
@
@ CELL table structure
@
@.....................................................................*/
typedef struct
	{
	USHORT  id;                   /* Cell id (from 1 to "gn_cell") */
	char    name[12];             /* Cell name */
	USHORT  enable;               /* Cell enabled(=1) or disabled(=0) by the operator */
	S_cfg_cell   s_cfg_cell;      /* configuration data */
	S_state s_state;              /* Automa status */
	ULONG   status;               /* status of the cell (bit table) */
	USHORT  cts_status;           /* status of the CTS function */
	USHORT  celcmd_status;        /* status of the Cell Command request */
	S_wk_cell   s_wk_cell;        /* working data */
	S_parcres   s_parcres;        /* parcel data on the cell */
	S_move      s_move;           /* data for unloading on the chute */
	S_recent_req   s_recent_req;  /* Info about last norm. recent. request */
	S_failure   s_failure;        /* Cell failure */
	S_cellstat  s_cellstat;       /* Statistic, diagnostic and status data */
	}S_cell;

/*@.....................................................................
@
@ CELL HEADER TABLE structure
@
@.....................................................................*/
typedef struct
	{
	USHORT  n_max_error;    /* maximum number of offloading error before
														 declaring the Cell in failure
														 65535 = never set the cell failure */
	}S_cell_header;

/*@.....................................................................
@
@ CELL table global(for JOB)
@
@.....................................................................*/

DEFDBGBLVAR(cell);
DEFDBGBLHEADER(cell);


/*@.....................................................................
@
@ General UTILITY routines prototypes relevant to the SCCELL table
@ defined in the UTILxxx.C library files
@
@.....................................................................*/
void  set_offl_error(S_cell *ps_cell_ele, USHORT error_type);
void  reset_offl_error(S_cell *ps_cell_ele, USHORT cell_error);
void  set_offloading_cnt(S_cell *ps_cell_ele, USHORT offset_ph0, USHORT n_chute,USHORT type);
void  write_parcel_result(S_cell *ps_cell_ele);
void  celcmd_reservation(USHORT n_cell,USHORT direction,long delay,USHORT profile,USHORT device);
void  celcmd_cancel(USHORT n_cell);
void  celcmd_reset_all(void);
void  send_redestination_message(S_cell *ps_cell_ele);
void  set_cell_empty_nok(S_cell *ps_cell_ele,USHORT event);
USHORT 	valid_chute(S_cell *ps_cell_ele,USHORT n_chute);
USHORT 	same_side_chute(S_cell *ps_cell_ele,USHORT n_chute);


/*@.....................................................................
@
@ End of CELL table definition
@
@.....................................................................*/
#endif
