/*@***************************************************************************
@
@       PROJECT         : XB: Sorter Control and Induction Station Control
@
@       FILE            : MESSAGE.C
@
@       VERSION         : XB5.0 and DDS2.0
@
@       TYPE            : LIBRARY
@
@       DESCRIPTION     : Message routines
@
@       COMPILER        : ic386
@
@       OPERATING SYSTEM: QNX / iRMX for Windows
@
@       NOTE            : completely translated into English
@										     	18/12/96 - The "send_mes" routine can displays
@                         trace informations - if requested -
@										     	27/05/97 - Formal update only
@                         Gen. 2001 prepared for QNX 
@
@***************************************************************************/

#include "xblib.h"


/*@**************************************************************************
@
@ Local defines
@
@***************************************************************************/

static S_id_token  s_id_token_mbx_logerr  = {"MBX_D_LOGERR",0};


/*@**************************************************************************
@
@ FUNCTION      : send_mes
@
@ DESCRIPTION   : it completes the message and sends it to a data mailbox
@
@ SYNOPSYS      : send_mes(code,subcode,mbx,l_mes,p_mes,src);
@
@ PARAMETERS    : UCHAR code=message code
@                 UCHAR subcode=message subcode
@                 TOKEN mbx=mailbox token
@                 USHORT l_mes=message length
@                 POINTER p_mes=message pointer
@                 TOKEN src=src_task
@
@ RETURN        : none
@
@ NOTE          : Abort task in case of error (macro CHK)
@
@***************************************************************************/
#define MAX_ACTIVE_MEX_TO_TRACE	gps_db->KN_area[INIT_MESSAGE_TO_TRACE_TABLE]
#define	TABLE_MEX_TO_TRACE(i)	gps_db->KN_area[INIT_MESSAGE_TO_TRACE_TABLE+i]

void send_mes(UCHAR code,UCHAR subcode,TOKEN mbx,USHORT l_mes,POINTER p_mes,TOKEN src)
{
USHORT status;
char message[512];
char m1[128],m2[128];
S_ERROR *ps_ERROR;
USHORT i;

	S_header_mes *ps_header_mes=(S_header_mes *)message;

	/* it sets the token of the last task transmitting the message */
	ps_header_mes->pv_src=src;

	/* it sets the token ( = id ) of the source task */
  ps_header_mes->src=get_task_id((POINTER)0);

	if (!is_valid_mbx(mbx))
		{
		sprintf(m1,"<send_mes>: invalid mailbox id 0x%x in the task 0x%x of the job 0x%x",
						mbx,
						ps_header_mes->src,
						(unsigned int)get_job_id());
		WRITE_TRACE(m1);
		return;
		}

	if (!s_id_token_mbx_logerr.id)
		{
		s_id_token_mbx_logerr.id = create_FIFO_D_mailbox(s_id_token_mbx_logerr.name);
#ifdef TEST_MBX_LOGERR
		sprintf(m1,"<send_mes>: mbx logerr initialized (token 0x%x) - task 0x%x",
						s_id_token_mbx_logerr.id,
						ps_header_mes->src);
		WRITE_TRACE(m1);
#endif
		}

	if (code==M_ERROR && mbx==s_id_token_mbx_logerr.id)
		{
		/* write the ERROR in the trace memory area (to have the
			 right error time) */
		ps_ERROR=(S_ERROR *)p_mes;

		switch(subcode)
			{
			case MS_ERROR_BEGIN:
				sprintf(m2,"ERROR BEGIN:     ");
				break;

			case MS_ERROR_END:
				sprintf(m2,"ERROR END: ");
				break;

			case MS_EMERG_BEGIN:
				sprintf(m2,"EMERGENCY BEGIN: ");
				break;

			case MS_EMERG_END:
				sprintf(m2,"EMERGENCY END: ");
				break;

			case MS_DEVICE_DISABLED:
				if (ps_ERROR->id_error!=ME_AUTO)
					sprintf(m2,"DEVICE DISABLED: ");
				else
					sprintf(m2,"DEVICE AUTO-DISABLED: ");
				break;

			case MS_DEVICE_ENABLED:
				if (ps_ERROR->id_error!=ME_AUTO)
					sprintf(m2,"DEVICE ENABLED: ");
				else
					sprintf(m2,"DEVICE AUTO-ENABLED: ");
				break;

			case MS_DEVICE_NOT_FOUND:
					sprintf(m2,"DEVICE NOT FOUND: ");
				break;

			case MS_ERROR_EVENT:
				sprintf(m2,"ERROR EVENT: ");
				break;

			default:
				sprintf(m2,"<send_mes> Message discarded for wrong subcode%d from task 0x%x",subcode,src);
				return;
			}

		sprintf(m1,"%s DEVICE = %2d, ID_DEVICE = %3d, ID_ERROR = %2d",
						m2,
						ps_ERROR->device,
						ps_ERROR->id_device,
						ps_ERROR->id_error);
		WRITE_TRACE(m1);
		}


	/* it sets the message code,subcode and length  */
	ps_header_mes->code=code;
	ps_header_mes->subcode=subcode;
	ps_header_mes->length=l_mes;
	if (l_mes)
		memcpy(&message[sizeof(S_header_mes)],p_mes,l_mes);


	/* CC 18/12/96 - It displays trace informations - if requested - */
	for (i=1;i<=MAX_ACTIVE_MEX_TO_TRACE;i++)
		{
		if (TABLE_MEX_TO_TRACE(i)==code)
			{
			int j,k,l;
			unsigned int c;
			char task_name[17];
			char mbx_name[17];
			memset(task_name,0,sizeof(task_name));
			memset(mbx_name,0,sizeof(mbx_name));
			strncpy(mbx_name,get_local_mailbox_name(mbx),sizeof(mbx_name)-1);
#ifndef QNX
			strncpy(task_name,get_local_task_name(ps_header_mes->src,get_job_id()),sizeof(task_name)-1);
			sprintf (m1,"Code=%d subcode=%d length=%d src=%04x-%s mbx=%04x-%s",
							code,
							subcode,
							l_mes,
							ps_header_mes->src,
							task_name,
							mbx,
							mbx_name);
#else
			sprintf (m1,"Code=%d subcode=%d length=%d job=%x src=%04x mbx=%04x-%s",
							code,
							subcode,
							l_mes,
							(unsigned int)get_job_id(),
							ps_header_mes->src,
							mbx,
							mbx_name);
#endif
			WRITE_TRACE(m1);
			if (l_mes)
				{
				j=0;
				k=0;
				for (l=0;l<l_mes;l++,p_mes++)
					{
					if (!k)
						{
						sprintf (m1,"Code=%d ->(%3d) ",code,j*20+1);
						j++;
						}
					c=*p_mes & 0xff;

					sprintf (m2,"%02x ",c);
					strcat (m1,m2);
					if (++k>=20)
						{
						k=0;
						WRITE_TRACE(m1);
						}
					}
				if (k)
					{
					WRITE_TRACE(m1);
					}
				}
			break;
			}
		}
	/* CC 18/12/96 - End of modification */

	mbx_data_send(mbx,message, (USHORT)(sizeof(S_header_mes)+l_mes));

}
/*@..........................................................................
@  end  of send_mes
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : get_internal_message_src
@
@ DESCRIPTION   : it returns the internal message source task
@
@ SYNOPSYS      : get_internal_message_src(message);
@
@ PARAMETERS    : POINTER message=internal message received
@
@ RETURN        : TOKEN src=message source task
@
@ NOTE          :
@
@***************************************************************************/
TOKEN get_internal_message_src(POINTER message)
{
	S_header_mes *ps_header_mes=(S_header_mes *)message;
	return(ps_header_mes->src);

}
/*@..........................................................................
@  end  of get_internal_message_src
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : get_internal_message_dest
@
@ DESCRIPTION   : it returns the internal message destination
@
@ SYNOPSYS      : get_internal_message_dest(message);
@
@ PARAMETERS    : POINTER message=internal message received
@
@ RETURN        : TOKEN dest=message destination
@
@ NOTE          :
@
@***************************************************************************/
USHORT get_internal_message_dest(POINTER message)
{
	S_header_mes *ps_header_mes=(S_header_mes *)message;
	return(ps_header_mes->dest);

}
/*@..........................................................................
@  end  of get_internal_message_dest
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : get_external_message_dest
@
@ DESCRIPTION   : it returns the external message dest
@
@ SYNOPSYS      : get_external_message_dest(message);
@
@ PARAMETERS    : POINTER message=external message received
@
@ RETURN        : USHORT dest=message destination
@
@ NOTE          :
@
@***************************************************************************/
USHORT get_external_message_dest(POINTER message)
{
	S_header_extmes *ps_header_extmes=(S_header_extmes *)message;
	return(ps_header_extmes->dest);

}
/*@..........................................................................
@  end  of get_external_message_dest
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : get_internal_message_code
@
@ DESCRIPTION   : it returns the internal message code
@
@ SYNOPSYS      : get_internal_message_code(message);
@
@ PARAMETERS    : POINTER message=internal message received
@
@ RETURN        : UCHAR code=message code
@
@ NOTE          :
@
@***************************************************************************/
UCHAR get_internal_message_code(POINTER message)
{
	S_header_mes *ps_header_mes=(S_header_mes *)message;
	return(ps_header_mes->code);

}
/*@..........................................................................
@  end  of get_internal_message_code
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : get_external_message_code
@
@ DESCRIPTION   : it returns the external message code
@
@ SYNOPSYS      : get_external_message_code(message);
@
@ PARAMETERS    : POINTER message=external message received
@
@ RETURN        : UCHAR code=message code
@
@ NOTE          :
@
@***************************************************************************/
UCHAR get_external_message_code(POINTER message)
{
	S_header_extmes *ps_header_extmes=(S_header_extmes *)message;
	return(ps_header_extmes->code);

}
/*@..........................................................................
@  end  of get_external_message_code
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : get_internal_message_subcode
@
@ DESCRIPTION   : it returns the internal message subcode
@
@ SYNOPSYS      : get_internal_message_subcode(message);
@
@ PARAMETERS    : POINTER message=internal message received
@
@ RETURN        : UCHAR subcode=message subcode
@
@ NOTE          :
@
@***************************************************************************/
UCHAR get_internal_message_subcode(POINTER message)
{
	S_header_mes *ps_header_mes=(S_header_mes *)message;
	return(ps_header_mes->subcode);

}
/*@..........................................................................
@  end  of get_internal_message_subcode
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : get_external_message_subcode
@
@ DESCRIPTION   : it returns the external message subcode
@
@ SYNOPSYS      : get_external_message_subcode(message);
@
@ PARAMETERS    : POINTER message=external message received
@
@ RETURN        : UCHAR subcode=message subcode
@
@ NOTE          :
@
@***************************************************************************/
UCHAR get_external_message_subcode(POINTER message)
{
	S_header_extmes *ps_header_extmes=(S_header_extmes *)message;
	return(ps_header_extmes->subcode);

}
/*@..........................................................................
@  end  of get_external_message_subcode
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : get_internal_message_length
@
@ DESCRIPTION   : it returns the internal message length
@
@ SYNOPSYS      : get_internal_message_length(message);
@
@ PARAMETERS    : POINTER message=internal message received
@
@ RETURN        : USHORT length=message length
@
@ NOTE          :
@
@***************************************************************************/
USHORT get_internal_message_length(POINTER message)
{
	S_header_mes *ps_header_mes=(S_header_mes *)message;
	return(ps_header_mes->length);

}
/*@..........................................................................
@  end  of get_internal_message_length
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : get_external_message_length
@
@ DESCRIPTION   : it returns the external message length
@
@ SYNOPSYS      : get_external_message_length(message);
@
@ PARAMETERS    : POINTER message=external message received
@
@ RETURN        : USHORT length=message length
@
@ NOTE          :
@
@***************************************************************************/
USHORT get_external_message_length(POINTER message)
{
	S_header_extmes *ps_header_extmes=(S_header_extmes *)message;
	return(ps_header_extmes->length);

}
/*@..........................................................................
@  end  of get_external_message_length
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : get_internal_message_text
@
@ DESCRIPTION   : it returns the address of the internal message text
@
@ SYNOPSYS      : get_internal_message_text(message);
@
@ PARAMETERS    : POINTER message=internal message received
@
@ RETURN        : POINTER text=internal message text structure pointer
@
@ NOTE          :
@
@***************************************************************************/
POINTER get_internal_message_text(POINTER message)
{
	return(&message[sizeof(S_header_mes)]);

}
/*@..........................................................................
@  end  of get_internal_message_text
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : get_external_message_text
@
@ DESCRIPTION   : it returns the external message text
@
@ SYNOPSYS      : get_external_message_text(message);
@
@ PARAMETERS    : POINTER message=external message received
@
@ RETURN        : POINTER text=external message text structure pointer
@
@ NOTE          :
@
@***************************************************************************/
POINTER get_external_message_text(POINTER message)
{
	return(&message[sizeof(S_header_extmes)]);

}
/*@..........................................................................
@  end  of get_external_message_text
@***************************************************************************/


