/*@***************************************************************************
@
@       PROJECT         : XB: Sorter Control and Induction Station Control
@
@       FILE            : QNXOBJ.C
@
@       VERSION         : XB5.0 and DDS2.0
@
@       TYPE            : LIBRARY
@
@
@       DESCRIPTION     : Routine to manage QNX object (or simulate iRMX object)
@
@       COMPILER        : ic386
@
@       OPERATING SYSTEM: QNX
@
@       NOTE            : completely translated into English
@
@***************************************************************************/
/* The following DEBUG_ALARM_TASK is for debug purpose only,
   to see if the AlarmTask is really called at every ms */
/*#define DEBUG_ALARM_TASK*/

#include "xblib.h"

#ifdef RMX
void dummy_qnxobj(void);
/*@**************************************************************************
@
@ FUNCTION      :  dummy_qnxobj
@
@ DESCRIPTION   :  dummy function (only to avoid compiling error)
@
@ SYNOPSYS      :  void dummy_qnxobj(void);
@
@ PARAMETERS    :  none
@
@ RETURN        :  none
@
@ NOTE          :  
@
@***************************************************************************/

void dummy_qnxobj(void)
{
}
/*@..........................................................................
@  end  of dummy_qnxobj
@***************************************************************************/
	
#endif

#ifdef QNX
/*@**************************************************************************
@
@ Local defines
@
@***************************************************************************/
/*#define TEST_LIB*/
/*#define TEST_OS_LIB*/
#define ALARM_TASK_PULSE_CODE _PULSE_CODE_MAXAVAIL - 1

void (*alarm_function)();
ULONG alarm_time_limit;
KN_FLAGS alarm_flags;

/*@**************************************************************************
@
@ Local function prototypes
@
@***************************************************************************/
void AlarmTask (void);

/*@**************************************************************************
@
@ FUNCTION      :  create_segment
@
@ DESCRIPTION   :  it creates a segment and clears it
@
@ SYNOPSYS      :  segment=create_segment (seg_size);
@
@ PARAMETERS    :  ULONG seg_size= segment size
@
@ RETURN        :  TOKEN segment=segment token
@
@ NOTE          :  Abort task in case of error 
@
@***************************************************************************/
TOKEN create_segment(ULONG seg_size)
{
	TOKEN segment;
	char m[128];
  DEFDBLOCVAR (catalog_seg);

  segment=0xFFFF;
  SCANDB(catalog_seg)
    {
    if (ps_catalog_seg_ele->id==0) 
      {
      ps_catalog_seg_ele->id = n_catalog_seg_wk + OBJ_SEG;
      segment = ps_catalog_seg_ele->id;
      sprintf(ps_catalog_seg_ele->name,"/SEG%d",ps_catalog_seg_ele->id);
      ps_catalog_seg_ele->size = seg_size;
			ps_catalog_seg_ele->seg_type=NORMALSEG;
	  	sprintf(m,"Segment created (%ld byte): 0x%x (Process owner=0x%x)",
              seg_size,
					    (unsigned int)segment ,
					    (unsigned int)get_job_id());
	  	WRITE_TRACE(m);
      break;
      }
    }

  if (segment==0xFFFF)
    {
	 	sprintf(m,"QNXOBJ:<create_segment> Segment: not created no space available (Thread owner=0x%4x)",
					  (USHORT)get_task_id((POINTER)0));
    CHK(m,segment);
    }

  return segment;

}
/*@..........................................................................
@  end  of create_segment
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :  delete_segment
@
@ DESCRIPTION   :  it deletes a segment
@
@ SYNOPSYS      :  delete_segment (segment);
@
@ PARAMETERS    :  TOKEN segment=segment token
@
@ RETURN        :  none
@
@ NOTE          :  
@
@***************************************************************************/
void delete_segment(TOKEN segment)
{
  DEFDBLOCVAR (catalog_seg);
	char m[128];

  ps_catalog_seg_ele = gps_catalog_seg + (segment - OBJ_SEG) ;

  if (ps_catalog_seg_ele->name[0]!=0)
    { 		
		shm_unlink(ps_catalog_seg_ele->name);

    memset(ps_catalog_seg_ele,0,sizeof(S_catalog_seg));
	  sprintf(m,"Deleted segment: 0x%4x (Process owner=0x%4x)",
					  (USHORT)segment & 0xFFFF,
					  (USHORT)get_job_id());
	  WRITE_TRACE(m);
   }

}
/*@..........................................................................
@  end  of delete_segment
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :  get_segment_size
@
@ DESCRIPTION   :  it gets the size of a segment
@
@ SYNOPSYS      :  size=get_segment_size (segment);
@
@ PARAMETERS    :  TOKEN segment=segment token
@
@ RETURN        :  ULONG size=segment size
@
@ NOTE          :  none
@
@***************************************************************************/
ULONG get_segment_size(TOKEN segment)
{
DEFDBLOCVAR (catalog_seg);

    ps_catalog_seg_ele = gps_catalog_seg + (segment - OBJ_SEG) ;
    if (ps_catalog_seg_ele->id==segment)
      return(ps_catalog_seg_ele->size);
  
  return(0);

}
/*@..........................................................................
@  end  of get_segment_size
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :  get_segment_name
@
@ DESCRIPTION   :  it gets the name of a segment
@
@ SYNOPSYS      :  get_segment_name (segment,name);
@
@ PARAMETERS    :  TOKEN segment = segment token
@
@ RETURN        :  char *name = segment name
@
@ NOTE          :  none
@
@***************************************************************************/
void get_segment_name(TOKEN segment,char *name)
{
DEFDBLOCVAR (catalog_seg);

  *name=0;
  SCANDB(catalog_seg)
    {
    if (ps_catalog_seg_ele->id==segment)
      {
      memcpy(name,ps_catalog_seg_ele->name,sizeof(ps_catalog_seg_ele->name));
			break;
      }
    }
  
}
/*@..........................................................................
@  end  of get_segment_name
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : buildptr
@
@ DESCRIPTION   : it creates a shared memory 
@
@ SYNOPSYS      : p=buildptr(segment,bUnmap);
@
@ PARAMETERS    : TOKEN segment= segment id
@									UCHAR bUnmap = 0 to avoid unmap, 1 to unmap
@
@ RETURN        : POINTER p   = memory address
@
@ NOTE          : 
@
@***************************************************************************/
POINTER buildptr(TOKEN segment,UCHAR bUnmap)
{
POINTER p;
char sShName [32];
TOKEN tid;
DEFDBLOCVAR(catalog_seg);
char m[128];

	if (segment<OBJ_SEG || segment>=OBJ_SEG+gn_catalog_seg)
		{
		sprintf(m,"QNXOBJ:<buildptr> Invalid segment=%d job=0x%lx task=0x%x",segment,get_job_id(),get_task_id((POINTER)0));
		CHK(m,E_EXIST);
		}
	ps_catalog_seg_ele=gps_catalog_seg + (segment - OBJ_SEG);

	/* GCE 20031203 unmap the last mapped segment before map a new one */
	if (bUnmap)
		{
		tid=get_task_id((POINTER)0);
		if (g_LastSegMapped[tid]!=(POINTER)0)
			{
			munmap(g_LastSegMapped[tid],g_LastSegSize[tid]);
			}
		}
		
	if (gps_catalog_ini[0]!=0)
		{
		sprintf(m,"QNXOBJ:<buildptr> gps_catalog corrupted: job=0x%lx task=0x%x",
							get_job_id(),
							tid);
		WRITE_TRACE(m);
		gps_catalog_ini[0]=0;
	  }
	
	/* CC 20030905 Create a dummy segment long 16 bytes if size is zero.
	   QNX6.2.1 doesn't allow size=0 */

	if (ps_catalog_seg_ele->size==0)
		ps_catalog_seg_ele->size=16;

	if (ps_catalog_seg_ele->name[0]!='/')
    sprintf(sShName ,"/%s",ps_catalog_seg_ele->name);
  else
    strncpy(sShName ,ps_catalog_seg_ele->name,sizeof(sShName ));

	p=map_sh_mem(sShName,ps_catalog_seg_ele->size);

	if (bUnmap)
		{
		if(ps_catalog_seg_ele->seg_type==BUFPOOLSEG)
			{
			g_LastSegMapped[tid]=p;
			g_LastSegSize[tid]=ps_catalog_seg_ele->size;
			}
    }

	return(p);
}
/*@..........................................................................
@  end  of buildptr
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : map_sh_mem
@
@ DESCRIPTION   : map a shared memory
@
@ SYNOPSYS      : p=map_sh_mem(sShName,size);
@
@ PARAMETERS    : POINTER sShName = segment name
@									ULONG    size		= segment size
@
@ RETURN        : POINTER p=segment address
@
@ NOTE          : Abort task in case of error (CHK macro)
@
@***************************************************************************/
POINTER map_sh_mem(POINTER sShName,ULONG size)
{
int fd;
POINTER p;
int bNew=0;

  fd = shm_open(sShName, O_RDWR , 0777);

  if (fd == -1) 
    {
	  fd = shm_open(sShName, O_RDWR | O_CREAT, 0777);
    if (fd == -1) 
      {
	    CHK("QNXOBJ:<map_sh_mem> Open failed ",errno);
      }
    bNew=1;
    }

  if (ftruncate(fd, size) == -1) 
  	{
    close(fd);
	  CHK("QNXOBJ:<map_sh_mem> ftruncate ",errno);
  	}
  	
  p = mmap(0, 
           size,
           PROT_READ | PROT_WRITE,
           MAP_SHARED, 
           fd, 
           0);
              
  close(fd);

  if (p == MAP_FAILED) 
    {
  	CHK("QNXOBJ:<map_sh_mem> mmap failed",errno);
    }

  if (bNew)
    {
    memset(p,0,size);
    }

	return(p);  

}

/*@**************************************************************************
@
@ FUNCTION      : create_FIFO_D_mailbox
@
@ DESCRIPTION   : it creates and catalogues a FIFO data mailbox
@
@ SYNOPSYS      : mailbox=create_FIFO_D_mailbox(name);
@
@ PARAMETERS    : POINTER name= mailbox name
@
@ RETURN        : TOKEN mailbox=mailbox token
@
@ NOTE          : Abort task in case of error (CHK macro)
@                 for QNX return the queue descriptor
@
@***************************************************************************/
TOKEN create_FIFO_D_mailbox(POINTER name)
{
	TOKEN object;

  object=create_mailbox(name);

  return(object);
}
/*@..........................................................................
@  end  of create_FIFO_D_mailbox
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : create_priority_D_mailbox
@
@ DESCRIPTION   : it creates and catalogues a priority data mailbox
@
@ SYNOPSYS      : mailbox=create_priority_D_mailbox(name);
@
@ PARAMETERS    : POINTER name= mailbox name
@
@ RETURN        : TOKEN mailbox=mailbox token
@
@ NOTE          : Abort task in case of error (CHK macro)
@
@***************************************************************************/
TOKEN create_priority_D_mailbox(POINTER name)
{
	TOKEN object;

	object=create_mailbox(name);

  return(object);

}
/*@..........................................................................
@  end  of create_priority_D_mailbox
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : create_FIFO_S_mailbox
@
@ DESCRIPTION   : it creates a FIFO segment mailbox
@
@ SYNOPSYS      : mailbox=create_FIFO_S_mailbox(name);
@
@ PARAMETERS    : POINTER name= mailbox name
@
@ RETURN        : TOKEN mailbox=mailbox token
@
@ NOTE          : Abort task in case of error (CHK macro)
@
@***************************************************************************/
TOKEN create_FIFO_S_mailbox(POINTER name)
{
	TOKEN object;

	object=create_mailbox(name);

  return(object);

}
/*@..........................................................................
@  end  of create_FIFO_S_mailbox
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : create_priority_S_mailbox
@
@ DESCRIPTION   : it creates a priority segment mailbox
@
@ SYNOPSYS      : mailbox=create_priority_S_mailbox(name);
@
@ PARAMETERS    : POINTER name= mailbox name
@
@ RETURN        : TOKEN mailbox=mailbox token
@
@ NOTE          : Abort task in case of error (CHK macro)
@
@***************************************************************************/
TOKEN create_priority_S_mailbox(POINTER name)
{
	TOKEN object;

	object=create_mailbox(name);

  return(object);

}
/*@..........................................................................
@  end  of create_priority_S_mailbox
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : create_mailbox
@
@ DESCRIPTION   : it creates a mailbox
@
@ SYNOPSYS      : mailbox=create_mailbox (flags);
@
@ PARAMETERS    : USHORT flags= mailbox flags
@
@ RETURN        : TOKEN mailbox=mailbox token
@
@ NOTE          : flags are not used in QNX (keep this function for compatibility)
@
@***************************************************************************/
TOKEN create_mailbox(POINTER name)
{
	TOKEN object;
	char m[128];
  char mbx_name[NAME_SIZE];
  DEFDBLOCVAR (catalog_mbx);
	mqd_t s_mqd_t;
  
  memset(mbx_name,0,sizeof(mbx_name));

  if (!*name)
    {
    sprintf(mbx_name,"/XX");
    }
  else
    {
    if (name[0]!='/')
      sprintf(mbx_name,"/%s",name);
    else
      sprintf(mbx_name,"%s",name);
    }
  
  object=0xFFFF;
  SCANDB(catalog_mbx)
    {
    if (!ps_catalog_mbx_ele->id
			|| !strcmp(mbx_name,ps_catalog_mbx_ele->name))
      {
      object = ps_catalog_mbx_ele->id;
      if (!ps_catalog_mbx_ele->id)
        ps_catalog_mbx_ele->id=n_catalog_mbx_wk + OBJ_MBX;

      if (!*name)
        {
        sprintf(mbx_name,"/MBX_%d",ps_catalog_mbx_ele->id  );
        }

 	    s_mqd_t = mq_open( mbx_name , 
	                O_RDWR | O_CREAT ,
	                O_RDWR ,
	                NULL );

      mq_close(s_mqd_t);
      sprintf(ps_catalog_mbx_ele->name,"%s",mbx_name);
      
      /* tests for catalog the mailbox */
      if (*name)  /* the mailbox must be named ! */
        {
        if (!object) /* test to catalog only new object */
          {
          catalog_mailbox_local(name,ps_catalog_mbx_ele->id);
          }
        }

      if (!object)
        {
	      sprintf(m,"MBx=%s created new object : 0x%x (Process owner=0x%x)",
                mbx_name,
					      (unsigned int)ps_catalog_mbx_ele->id,
					      (unsigned int)get_job_id());
        }
      else
        {
	      sprintf(m,"MBx=%s created : 0x%x (Process owner=0x%x)",
                mbx_name,
					      (unsigned int)object,
					      (unsigned int)get_job_id());
        }

	    WRITE_TRACE(m);
      object = ps_catalog_mbx_ele->id;
	    break;
      }
    }

  if (object==0xFFFF)
    {
	  sprintf(m,"MBX >%s<: not created no space available (Thread owner=0x%4x)",
						name,
					  (USHORT)get_task_id((POINTER)0));
	  CHK(m,object);
    }

	return(object);

}
/*@..........................................................................
@  end  of create_mailbox
@***************************************************************************/


/*@**************************************************************************
@
@ FUNCTION      : delete_mailbox
@
@ DESCRIPTION   : it delete a mailbox
@
@ SYNOPSYS      : delete_mailbox(mbx);
@
@ PARAMETERS    : TOKEN mailbox = mailbox token
@
@ RETURN        : 
@
@ NOTE          : 
@
@***************************************************************************/
void delete_mailbox(TOKEN mailbox)
{
DEFDBLOCVAR (catalog_mbx);

  ps_catalog_mbx_ele= gps_catalog_mbx + (mailbox - OBJ_MBX);

  if (ps_catalog_mbx_ele->id==mailbox)
    {
    memset(ps_catalog_mbx_ele,0,sizeof(S_catalog_mbx));
    }

}
/*@..........................................................................
@  end  of delete_mailbox
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : create_FIFO_region
@
@ DESCRIPTION   : it creates a FIFO region
@
@ SYNOPSYS      : region=create_FIFO_region(name);
@
@ PARAMETERS    : POINTER name= region name
@
@ RETURN        : TOKEN region= region token
@
@ NOTE          : Abort task in case of error (CHK macro)
@
@***************************************************************************/
TOKEN create_FIFO_region(POINTER reg_name)
{
	TOKEN object;
	object=create_region(reg_name);

  if (!*reg_name)
		{
		catalog_object_unnamed("FIFO region",object);
		}
	else 
		{
		catalog_region_local(reg_name,object);
		}

	return(object);
}
/*@..........................................................................
@  end  of create_FIFO_region
@***************************************************************************/
/*@**************************************************************************
@
@ FUNCTION      : create_priority_region
@
@ DESCRIPTION   : it creates a priority region
@
@ SYNOPSYS      : region=create_priority_region(name);
@
@ PARAMETERS    : POINTER name= region name
@
@ RETURN        : TOKEN region= region token
@
@ NOTE          : 
@
@***************************************************************************/
TOKEN create_priority_region(POINTER reg_name)
{
	TOKEN object;
	object=create_region(reg_name);

  if (!*reg_name)
		{
		catalog_object_unnamed("PRIORITY region",object);
		}
	else 
		{
		catalog_region_local(reg_name,object);
		}

	return(object);
} 
/*@..........................................................................
@  end  of create_priority_region
@***************************************************************************/
/*@**************************************************************************
@
@ FUNCTION      : create_region
@
@ DESCRIPTION   : it creates a region
@
@ SYNOPSYS      : region=create_region(name);
@
@ PARAMETERS    : POINTER name= region name
@
@ RETURN        : TOKEN region= region token
@
@ NOTE          : 
@
@***************************************************************************/
TOKEN create_region(POINTER reg_name)
{
	TOKEN object;
  DEFDBLOCVAR (catalog_reg);
	char m[128];

  object=-1;
  /* look for free location */
  SCANDB(catalog_reg)
    {
    if (!ps_catalog_reg_ele->id
			|| !strcmp(reg_name,ps_catalog_reg_ele->name))
      {
      ps_catalog_reg_ele->id = n_catalog_reg_wk + OBJ_REG;
      object = ps_catalog_reg_ele->id;
      /* save name */
      sprintf(ps_catalog_reg_ele->name,"%s",reg_name);
      /* mutex init. */

      ps_catalog_reg_ele->pmutex.count=_NTO_SYNC_NONRECURSIVE;
      ps_catalog_reg_ele->pmutex.owner=_NTO_SYNC_INITIALIZER;

	    sprintf(m,"Region/mutex %s created : 0x%x (Process owner=0x%x)",
              reg_name,
					    (unsigned int)object,
					    (unsigned int)get_job_id());
	    WRITE_TRACE(m);
	    break;
      }
    }

  /* return id */
	return(object);

}
/*@..........................................................................
@  end  of create_priority_region
@***************************************************************************/


/*@**************************************************************************
@
@ FUNCTION      : lock_db_mutex
@
@ DESCRIPTION   : it locks a priority region
@
@ SYNOPSYS      : lock_db_mutex(region);
@
@ PARAMETERS    : TOKEN region = region id
@
@ RETURN        : 
@
@ NOTE          : 
@
@***************************************************************************/
void lock_db_mutex(TOKEN region)
{
	int res=0;
  DEFDBLOCVAR (catalog_reg);
	char m[128];

  res=0xFFFF;
  /* look for free location */
  
  ps_catalog_reg_ele = gps_catalog_reg + (region - OBJ_REG);

  if ( ps_catalog_reg_ele->id==region)
    {
    res=pthread_mutex_lock( &ps_catalog_reg_ele->pmutex );
    ps_catalog_reg_ele->pid = get_job_id();
    }

  if (res!=E_OK)
    {
    sprintf(m,"QNXOBJ:<lock_db_mutex> db not locked");
 		CHK(m,res);
    }
}
/*@..........................................................................
@  end  of lock_db_mutex
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : unlock_all_db
@
@ DESCRIPTION   : it unlocks all region locked by the current process 
@
@ SYNOPSYS      : unlock_all_db();
@
@ PARAMETERS    : 
@
@ RETURN        : 
@
@ NOTE          : 
@
@***************************************************************************/
void unlock_all_db()
{
	pid_t pid;
  DEFDBLOCVAR (catalog_reg);

  pid=get_job_id();
  /* look for free location */
  SCANDB(catalog_reg)
    {
    if ( ps_catalog_reg_ele->pid==pid)
      {
      pthread_mutex_unlock( &ps_catalog_reg_ele->pmutex );
      }
    if ( ps_catalog_reg_ele->pid==0)
      break;
    }

}
/*@..........................................................................
@  end  of unlock_all_db
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      : unlock_db_mutex
@
@ DESCRIPTION   : it unlocks a priority region
@
@ SYNOPSYS      : unlock_db_mutex(region);
@
@ PARAMETERS    : TOKEN regine = region id
@
@ RETURN        : 
@
@ NOTE          : 
@
@***************************************************************************/
void unlock_db_mutex(TOKEN region)
{
	int res=0;
  DEFDBLOCVAR (catalog_reg);
	char m[128];

  res=0;
  /* look for free location */

  ps_catalog_reg_ele = gps_catalog_reg + (region - OBJ_REG);

  if ( ps_catalog_reg_ele->id==region)
    {
    pthread_mutex_unlock( &ps_catalog_reg_ele->pmutex );
    ps_catalog_reg_ele->pid = 0;
    res=1;
    }

  if (!res)
    {
    sprintf(m,"db not unlocked (region not found)");
	  WRITE_TRACE(m);
    }
}
/*@..........................................................................
@  end  of unlock_db_mutex
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       create_FIFO_semaphore
@
@ DESCRIPTION   :       it creates a FIFO semaphore
@
@ SYNOPSYS      :       semaphore=create_FIFO_semaphore (name,ini_value,max_value);
@
@ PARAMETERS    :       POINTER name= semaphore name
@                       USHORT ini_value= initial number of units
@                       USHORT max_value= maximum number of units
@
@ RETURN        :       TOKEN semaphore=semaphore token
@
@ NOTE          :       Abort task in case of error (CHK macro)
@
@***************************************************************************/
TOKEN create_FIFO_semaphore(POINTER name,USHORT ini_value,USHORT max_value)
{
  DEFDBLOCVAR (catalog_sem);
	char m[128];
	int res=0;

  if (*name)
		{
    /* look if the sem exist */
    SCANDB(catalog_sem)
      {
      if (!ps_catalog_sem_ele->id)
        break;

      if ( !strcmp(name,ps_catalog_sem_ele->name))
        {
        /* the sem exist return its id */
        return (ps_catalog_sem_ele->id);
        }
      }
    /* the sem does not exist go ahead */
    }

  SCANDB(catalog_sem)
    {
    /* look for an empty record */
    if ( ps_catalog_sem_ele->id==0 )
      {
      /* create a new sem */
      res=sem_init(&ps_catalog_sem_ele->sem , 1 , ini_value);
      if (res!=E_OK)
        {
        memset(ps_catalog_sem_ele,0,sizeof(S_catalog_sem));
        sprintf(m,"QNXOBJ:<create_FIFO_semaphore> Impossible create semaphore (%s)",
					      name);
        CHK(m,res);
        }
      else
        {
        ps_catalog_sem_ele->id=n_catalog_sem_wk + OBJ_SEM;
        if (max_value==0xFFFF)
	        ps_catalog_sem_ele->nULimit=0;
        else
	        ps_catalog_sem_ele->nULimit=max_value;
        memset(ps_catalog_sem_ele->name,0,sizeof(ps_catalog_sem_ele->name));
        if (!*name)
		      {
		      catalog_object_unnamed("FIFO semaphore",ps_catalog_sem_ele->id);
          sprintf(ps_catalog_sem_ele->name,"SEM_%d",ps_catalog_sem_ele->id);
		      }
	      else 
		      {
		      catalog_semaphore_local(name,ps_catalog_sem_ele->id);
          strncpy(ps_catalog_sem_ele->name,name,sizeof(ps_catalog_sem_ele->name));
		      }
        return (ps_catalog_sem_ele->id);
        }
      }
    }
  return(0);

}
/*@..........................................................................
@  end  of create_FIFO_semaphore
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       create_normal_task
@
@ DESCRIPTION   :       it creates a normal task
@
@ SYNOPSYS      :       task=create_normal_task(name,task,priority);
@
@ PARAMETERS    :       POINTER name= task name
@                       void(far *)taskname(void) = task name
@                       UCHAR priority = task priority
@
@ RETURN        :       TOKEN task= task token
@
@ NOTE          :       Abort task in case of error (CHK macro)
@
@***************************************************************************/
TOKEN create_normal_task(POINTER name,void (far *taskname)(void ),UCHAR priority)
{
	int res=0;
	char m[128];
  DEFDBLOCVAR (catalog_tsk);
  
	pthread_t object;
  pthread_attr_t attr;

  pthread_attr_init( &attr );
  res=pthread_create( &object, &attr , (void * (*)(void*)) taskname, NULL );
  if (res!=E_OK)
    {
    sprintf(m,"QNXOBJ:<create_normal_task> Impossible create task %s",
			      name);
    CHK(m,res);
    }
#ifdef TEST_LIB
  else
    {
    sprintf(m,"QNXOBJ:<create_normal_task> Task %s created !" ,
			      name);
    CHK(m,res);
    }
#endif
  
  change_task_priority(object,0,priority);

	if (*name)
		catalog_task_local(name,object);
	else
		catalog_object_unnamed("normal task",object);

	return(object);

}
/*@..........................................................................
@  end  of create_normal_task
@***************************************************************************/


/*@**************************************************************************
@
@ FUNCTION      :       suspend_task
@
@ DESCRIPTION   :       it suspends a task
@
@ SYNOPSYS      :       suspend_task(task);
@
@ PARAMETERS    :       TOKEN task = task to suspend (0=itself)
@
@ RETURN        :       none
@
@ NOTE          :       Abort task in case of error (CHK macro)
@
@***************************************************************************/
void suspend_task(TOKEN task)
{
	char m[128];
  
  /* used to suspend the current task */
  if (!task)
    {
		/* set the minmum priority */
		change_priority(MIN_PRIORITY);
    pause();
    }
  else
    {
    sprintf(m,"Request to suspend task %d (not FORESEEN in this library) \n",task);
		WRITE_TRACE(m);
    }

}
/*@..........................................................................
@  end  of suspend_task
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       resume_task
@
@ DESCRIPTION   :       it resumes a task
@
@ SYNOPSYS      :       resume_task(task);
@
@ PARAMETERS    :       TOKEN task = task to resume
@
@ RETURN        :       none
@
@ NOTE          :       Abort task in case of error (CHK macro)
@
@***************************************************************************/
void resume_task(TOKEN task)
{
	char m[128];

  sprintf(m,"Request to resume task %d (not FORESEEN in this library) \n",task);
	WRITE_TRACE(m);

}
/*@..........................................................................
@  end  of resume_task
@***************************************************************************/
/*@**************************************************************************
@
@ FUNCTION      :       delete_task
@
@ DESCRIPTION   :       it deletes a task
@
@ SYNOPSYS      :       delete_task(task);
@
@ PARAMETERS    :       TOKEN task = task to delete (0=itself)
@
@ RETURN        :       none
@
@ NOTE          :       none
@
@***************************************************************************/
void delete_task(TOKEN task)
{
  DEFDBLOCVAR (catalog_tsk);
  int res=0;
	char m[128];
  
  if (task)
    {
    SCANDB(catalog_tsk)
      {
      if ( ps_catalog_tsk_ele->id==task)
        {
        res=pthread_abort(ps_catalog_tsk_ele->thread);
        if (res!=E_OK)
          {
          sprintf(m,"Delete thread failed (%d)",task);
          WRITE_TRACE(m);
          }
        else
  	      {
     		  memset(ps_catalog_tsk_ele,0,sizeof(S_catalog_tsk));
      	  }
        break;
        }
      }
    }
  else
    {
    pthread_exit(NULL);
    }

}
/*@..........................................................................
@  end  of delete_task
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       get_task_id
@
@ DESCRIPTION   :       it gets the task id of a task
@
@ SYNOPSYS      :       id=get_task_id(name);
@
@ PARAMETERS    :       POINTER name = task name (NULL=itself)
@
@ RETURN        :       TOKEN id = task id
@
@ NOTE          :       Abort task in case of error (CHK macro)
@
@***************************************************************************/
TOKEN get_task_id(POINTER name)
{
	TOKEN id;
	char m[128];
  DEFDBLOCVAR (catalog_tsk);
  
  if (!name)
		{
    id=(USHORT)pthread_self();
    }
  else
		{
    SCANDB(catalog_tsk)
      {
      if ( !strcmp(name,ps_catalog_tsk_ele->name ))
        {
        id=ps_catalog_tsk_ele->tsk_id;
        break;
        }
      }
    }

	return(id);
}
/*@..........................................................................
@  end  of get_task_id
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       delete_job
@
@ DESCRIPTION   :       it deletes a job
@
@ SYNOPSYS      :       delete_job(job);
@
@ PARAMETERS    :       KN_TOKEN job= job to delete (0=itself)
@
@ RETURN        :       none
@
@ NOTE          :       used to delete the current process (job==0)
@
@***************************************************************************/
void delete_job(KN_TOKEN job)
{

	if (!job)
    {
    kill(get_job_id(),0);
    }

}
/*@..........................................................................
@  end  of delete_job
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       get_job_id
@
@ DESCRIPTION   :       it gets the job id
@
@ SYNOPSYS      :       id=get_job_id();
@
@ PARAMETERS    :       none
@
@ RETURN        :       TOKEN id = job id
@
@ NOTE          :       Abort task in case of error (CHK macro)
@
@***************************************************************************/
KN_TOKEN get_job_id(void) /* should return long */
{

  return(getpid());

}
/*@..........................................................................
@  end  of get_job_id
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :  sm_send
@
@ DESCRIPTION   :  it sends unit=units to a semaphore
@
@ SYNOPSYS      :  sm_send(semaphore,units);
@
@ PARAMETERS    :  TOKEN   semaphore = semaphore token
@                  USHORT  units = units requested
@
@ RETURN        :  none
@
@ NOTE          :  Abort task in case of error (CHK macro)
@
@***************************************************************************/
void sm_send(TOKEN semaphore,USHORT units)
{
  DEFDBLOCVAR (catalog_sem);
	char m[128];
	int res=EINVAL;
  int i;
  int value;

  ps_catalog_sem_ele = gps_catalog_sem + (semaphore - OBJ_SEM);

	if (ps_catalog_sem_ele->id==semaphore)
    {
  	sem_getvalue(&ps_catalog_sem_ele->sem,&value);
    if ( (value + units) <= ps_catalog_sem_ele->nULimit || ps_catalog_sem_ele->nULimit==0 )
      {
	    for (i=units;i;i--)
 	     {
   	   res=sem_post(&ps_catalog_sem_ele->sem);
#ifdef TEST_LIB
        sprintf(m,"Posted 1 unit %s : %d",ps_catalog_sem_ele->name,units);
        WRITE_TRACE(m);
#endif
				if (res!=EOK )
					{
					CHK("<sm_send> fail (sem_post error)",res);
					}
        }
      }
		}
	else
		{
		CHK("<sm_send> fail (Semaphore not found)",res);
		}


}
/*@..........................................................................
@  end  of sm_send
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       sm_binary_wait_forever
@
@ DESCRIPTION   :       it waits forever on a semaphore for 1 unit
@
@ SYNOPSYS      :       units=sm_binary_wait_forever(semaphore);
@
@ PARAMETERS    :       TOKEN   semaphore = semaphore token
@
@ RETURN        :       USHORT units=number of units
@
@ NOTE          :       Abort task in case of error (CHK macro)
@
@***************************************************************************/
USHORT sm_binary_wait_forever(TOKEN semaphore)
{
  DEFDBLOCVAR (catalog_sem);

  ps_catalog_sem_ele = gps_catalog_sem + (semaphore - OBJ_SEM);

  if (ps_catalog_sem_ele->id==semaphore)
    {
    sem_wait(&ps_catalog_sem_ele->sem);
		}
  else
    {
		CHK("<sm_binary_wait_forever> fail (Semaphore not found)",EINVAL);
    }

  return(1);
}
/*@..........................................................................
@  end  of sm_binary_wait_forever
@***************************************************************************/
/*@**************************************************************************
@
@ FUNCTION      :       sm_binary_wait
@
@ DESCRIPTION   :       it waits on a semaphore for 1 unit end for time=time_limit
@
@ SYNOPSYS      :       unints=sm_binary_wait(semaphore,time_limit);
@
@ PARAMETERS    :       TOKEN   semaphore = semaphore token
@                       USHORT  time_limit = time to wait
@
@ RETURN        :       USHORT result: number of units (0=time elapsed)
@
@ NOTE          :       Abort task in case of error (CHK macro)
@
@***************************************************************************/
USHORT sm_binary_wait(TOKEN semaphore,USHORT time_limit)
{
DEFDBLOCVAR (catalog_sem);
  
  ps_catalog_sem_ele = gps_catalog_sem + (semaphore - OBJ_SEM);
	if (ps_catalog_sem_ele->id==semaphore)
    {
    FOREVER
      {
      if (!sem_trywait(&ps_catalog_sem_ele->sem))
        {
        return(1);
        }

      if (!time_limit)
        break;
      
      if (time_limit != WAIT_FOREVER)
        {
				time_limit--;				/* wait 1 TICK=for 10 ms each time */
        }

      usleep(10000);
      }
		}
  else
    {
		CHK("<sm_binary_wait> fail (Semaphore not found)",EINVAL);
    }
  
  return(0);

}
/*@..........................................................................
@  end  of sm_binary_wait_time
@***************************************************************************/
/*@**************************************************************************
@
@ FUNCTION      :       sm_binary_nowait
@
@ DESCRIPTION   :       it tests a semaphore for 1 unit
@
@ SYNOPSYS      :       units=sm_binary_nowait(semaphore);
@
@ PARAMETERS    :       TOKEN   semaphore = semaphore token
@
@ RETURN        :       USHORT units=number of units
@
@ NOTE          :       Abort task in case of error (CHK macro)
@
@***************************************************************************/
USHORT sm_binary_nowait(TOKEN semaphore)
{
  DEFDBLOCVAR (catalog_sem);

  ps_catalog_sem_ele = gps_catalog_sem + (semaphore - OBJ_SEM);
	if (ps_catalog_sem_ele->id==semaphore)
    {
    if (!sem_trywait(&ps_catalog_sem_ele->sem))
      return(1);
    else
      return(0);
    }
  else
    {
		CHK("<sm_binary_nowait> fail (Semaphore not found)",EINVAL);
    }

  return(0);

}
/*@..........................................................................
@  end  of sm_binary_nowait
@***************************************************************************/


/*@**************************************************************************
@
@ FUNCTION      :       sm_wait_forever
@
@ DESCRIPTION   :       it waits forever on a semaphore for unit=units
@
@ SYNOPSYS      :       units=sm_wait_forever(semaphore,units);
@
@ PARAMETERS    :       TOKEN   semaphore = semaphore token
@                       USHORT  units = units requested
@
@ RETURN        :       USHORT units=number of units
@
@ NOTE          :       Abort task in case of error (CHK macro)
@
@***************************************************************************/
USHORT sm_wait_forever(TOKEN semaphore,USHORT units)
{
DEFDBLOCVAR (catalog_sem);
char m[128];
int value=0;
  
  ps_catalog_sem_ele = gps_catalog_sem + (semaphore - OBJ_SEM);

	if (ps_catalog_sem_ele->id==semaphore)
    {
    FOREVER
      {
      while (!sem_trywait(&ps_catalog_sem_ele->sem))
        {
        value+=1;
        if (value==units)
          return(value);
        }

      /* wait for 10 ms each time */
      usleep(10000);
      }
		}
  else
    {
		CHK("<sm_wait_forever> fail (Semaphore not found)",EINVAL);
    }

  return(0); 
}
/*@..........................................................................
@  end  of sm_wait_forever
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       sm_nowait
@
@ DESCRIPTION   :       it tests a semaphore for unit=units
@
@ SYNOPSYS      :       units=sm_nowait(semaphore,units);
@
@ PARAMETERS    :       TOKEN   semaphore = semaphore token
@                       USHORT  units = units requested
@
@ RETURN        :       USHORT units=number of units
@
@ NOTE          :       Abort task in case of error (CHK macro)
@
@***************************************************************************/
USHORT sm_nowait(TOKEN semaphore,USHORT units)
{
	return(sm_wait(semaphore,0,units));
}
/*@..........................................................................
@  end  of sm_nowait
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       sm_wait
@
@ DESCRIPTION   :       it waits on a semaphore for unit=units end for time=time_limit
@
@ SYNOPSYS      :       units=sm_wait(semaphore,time_limit,units);
@
@ PARAMETERS    :       TOKEN   semaphore = semaphore token
@                       USHORT  time_limit = time to wait
@                       USHORT  units = units requested
@
@ RETURN        :       USHORT units: number of units (0=time elapsed)
@
@ NOTE          :       Abort task in case of error (CHK macro)
@
@***************************************************************************/
USHORT sm_wait(TOKEN semaphore,USHORT time_limit,USHORT units)
{
DEFDBLOCVAR (catalog_sem);
char m[128];
int value=0;
  
  ps_catalog_sem_ele = gps_catalog_sem + (semaphore - OBJ_SEM);

	if (ps_catalog_sem_ele->id==semaphore)
    {
    FOREVER
      {
      while (!sem_trywait(&ps_catalog_sem_ele->sem))
        {
        value+=1;
        if (value==units)
          return(value);
        }

      if (!time_limit)
        break;
      
      if (time_limit != WAIT_FOREVER)
        {
				time_limit--;				/* wait 1 TICK=for 10 ms each time */
        }

      usleep(10000);
      }
		}
  else
    {
		CHK("<sm_wait> fail (Semaphore not found)",EINVAL);
    }
  
  return(0);

}
/*@..........................................................................
@  end  of sm_wait
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       mbx_data_send
@
@ DESCRIPTION   :       it sends a message to a data mailbox
@
@ SYNOPSYS      :       actual=mbx_data_send(mailbox,p_data,l_data);
@
@ PARAMETERS    :       TOKEN   mailbox = data mailbox token
@                       POINTER p_data = pointer to data buffer
@                       USHORT  l_data = length of message
@
@ RETURN        :       none
@
@ NOTE          :       
@
@***************************************************************************/
void mbx_data_send(TOKEN mailbox,POINTER p_data,USHORT l_data)
{
DEFDBLOCVAR (catalog_mbx);
mqd_t s_mqd_t;
char m[128];
struct mq_attr mqstat;
/*UCHAR priority;*/
struct  sched_param param;

  SchedGet(0,0,&param);
  param.sched_priority++;
  SchedSet(0,0,SCHED_FIFO ,&param);
	/* 
  priority=get_task_priority(0,0);  
  change_task_priority(0,0,priority+1);
	*/

  ps_catalog_mbx_ele = gps_catalog_mbx + (mailbox - OBJ_MBX);

  if ( ps_catalog_mbx_ele->id == mailbox )
    {
    /* open mbx */
 	  s_mqd_t = mq_open(ps_catalog_mbx_ele->name, 
	                  O_WRONLY ,
	                  0 ,
	                  NULL );
    if (s_mqd_t == -1)
      {
      sprintf(m,"QNXOBJ:<mbx_data_send> Impossible open mbx >%s< (%d), job 0x%x",
				      ps_catalog_mbx_ele->name,
							mailbox,
							(unsigned int)get_job_id());
      CHK(m,errno);
      }

    mq_getattr(s_mqd_t,
	               &mqstat);
    if (mqstat.mq_curmsgs>1000)
      {
      sprintf(m,"QNXOBJ:<mbx_data_send> Impossible send msg mbx >%s<(%d) full, job 0x%x",
				      ps_catalog_mbx_ele->name,
							mailbox,
							(unsigned int)get_job_id());
      WRITE_TRACE(m);
      }
    else
      {
	    /* send message to queue */
		  mq_send(s_mqd_t,
	   	       p_data,
	      	    l_data,
	         	 0);
      }

    mq_close(s_mqd_t);
    }
    
	/*    
  change_task_priority(0,0,priority);
	*/

  param.sched_priority--;
  SchedSet(0,0,SCHED_FIFO ,&param);

}
/*@..........................................................................
@  end  of mbx_data_send
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       mbx_data_wait
@
@ DESCRIPTION   :       it waits on a data mailbox for time=time_limit
@
@ SYNOPSYS      :       actual=mbx_data_wait(mailbox,p_data,l_data,time_limit);
@
@ PARAMETERS    :       TOKEN   mailbox = data mailbox token
@                       POINTER p_data = pointer to data buffer
@                       USHORT  time_limit = time to wait
@
@ RETURN        :       USHORT actual=number of bytes received (0=time elapsed)
@
@ NOTE          :       
@
@***************************************************************************/
USHORT mbx_data_wait(TOKEN mailbox,POINTER p_data,USHORT time_limit)
{
USHORT l_mex;
DEFDBLOCVAR (catalog_mbx);
mqd_t s_mqd_t;
struct mq_attr mqstat;
char m[128];
/*UCHAR priority;*/
struct  sched_param param;

  SchedGet(0,0,&param);
  param.sched_priority++;
  SchedSet(0,0,SCHED_FIFO ,&param);
	/* 
  priority=get_task_priority(0,0);  
  change_task_priority(0,0,priority+1);
	*/
  
  ps_catalog_mbx_ele = gps_catalog_mbx + (mailbox - OBJ_MBX);

  if ( ps_catalog_mbx_ele->id == mailbox )
    {
    /* open mbx */
 	  s_mqd_t = mq_open(ps_catalog_mbx_ele->name, 
	                  O_RDONLY,
	                  0 ,
	                  NULL );
    if (s_mqd_t == -1)
      {
      sprintf(m,"QNXOBJ:<mbx_data_wait> Impossible open mbx >%s< (%d), job 0x%x",
				      ps_catalog_mbx_ele->name,
							mailbox,
							(unsigned int)get_job_id());
      CHK(m,errno);
      }

    FOREVER
      {
	    mq_getattr(s_mqd_t,
	               &mqstat);
      if (mqstat.mq_curmsgs)
        {
	      l_mex = mq_receive(s_mqd_t, 
	                       p_data, 
	                       mqstat.mq_msgsize,
	                       NULL);
        mq_close(s_mqd_t);
	/*    
  change_task_priority(0,0,priority);
	*/

  param.sched_priority--;
  SchedSet(0,0,SCHED_FIFO ,&param);
       return(l_mex);
        }
      if (!time_limit)
        break;
      else
				time_limit--;				/* wait 1 TICK=for 10 ms each time */

      usleep(10000);
      }
    mq_close(s_mqd_t);
		}

  /*    
  change_task_priority(0,0,priority);
	*/

  param.sched_priority--;
  SchedSet(0,0,SCHED_FIFO ,&param);

	return(0);
}
/*@..........................................................................
@  end  of mbx_data_wait
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       mbx_data_wait_forever
@
@ DESCRIPTION   :       it waits forever a data mailbox
@
@ SYNOPSYS      :       actual=mbx_data_wait_forever(mailbox,p_data,l_data);
@
@ PARAMETERS    :       TOKEN   mailbox = data mailbox token
@                       POINTER p_data = pointer to data buffer
@
@ RETURN        :       USHORT actual=number of bytes received
@
@ NOTE          :       Abort task in case of error (CHK macro)
@
@***************************************************************************/
USHORT mbx_data_wait_forever(TOKEN mailbox,POINTER p_data)
{
DEFDBLOCVAR (catalog_mbx);
mqd_t s_mqd_t;
struct mq_attr mqstat;
int len;
char m[128];
/*UCHAR priority;*/
struct  sched_param param;

  SchedGet(0,0,&param);
  param.sched_priority++;
  SchedSet(0,0,SCHED_FIFO ,&param);
	/* 
  priority=get_task_priority(0,0);  
  change_task_priority(0,0,priority+1);
	*/

  ps_catalog_mbx_ele = gps_catalog_mbx + (mailbox - OBJ_MBX);

  if ( ps_catalog_mbx_ele->id == mailbox )
    {
    /* open mbx */
 	  s_mqd_t = mq_open(ps_catalog_mbx_ele->name, 
	                  O_RDONLY,
	                  0,
	                  NULL );
    if (s_mqd_t == -1)
      {
      sprintf(m,"QNXOBJ:<mbx_data_wait_forever> Impossible open mbx >%s< (%d), job 0x%x",
				      ps_catalog_mbx_ele->name,
							mailbox,
							(unsigned int)get_job_id());
      CHK(m,errno);
      }
	  mq_getattr(s_mqd_t,
	             &mqstat);
    /* get message to queue */
	  len = mq_receive(s_mqd_t, 
	                   p_data, 
	                   mqstat.mq_msgsize,
	                   NULL);
    mq_close(s_mqd_t);
	/*    
  change_task_priority(0,0,priority);
	*/

  param.sched_priority--;
  SchedSet(0,0,SCHED_FIFO ,&param);

    return(len);
    }

  /*  
  change_task_priority(0,0,priority);
	*/

  param.sched_priority--;
  SchedSet(0,0,SCHED_FIFO ,&param);

  return (0);
}
/*@..........................................................................
@  end  of mbx_data_wait_forever
@***************************************************************************/


/*@**************************************************************************
@
@ FUNCTION      :       mbx_object_send
@
@ DESCRIPTION   :       it sends a message to a object mailbox
@
@ SYNOPSYS      :       mbx_object_send(mailbox,object);
@
@ PARAMETERS    :       TOKEN   mailbox = segment mailbox token
@                       TOKEN   object = object to be sent
@
@ RETURN        :       none
@
@ NOTE          :       for QNX use mbx_object_send_reply with replay=0
@
@***************************************************************************/
void mbx_object_send(TOKEN mailbox,TOKEN object)
{

  mbx_object_send_reply(mailbox,object,0);

}
/*@..........................................................................
@  end  of mbx_object_send
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       mbx_object_send_reply
@
@ DESCRIPTION   :       it sends a message to a object mailbox with reply
@
@ SYNOPSYS      :       mbx_object_send_reply(mailbox,object,reply);
@
@ PARAMETERS    :       TOKEN   mailbox = segment mailbox token
@                       TOKEN   object = object to be sent
@                       TOKEN   reply = mailbox or semaphore token where send the response
@
@ RETURN        :       none
@
@ NOTE          :       for QNX send the object token value and the reply semaphore token
@ 2003/05/28 CC The task that send the object message must not be blocked 
@ (increase proper priority by 1)
@
@***************************************************************************/
void mbx_object_send_reply(TOKEN mailbox,TOKEN object,TOKEN reply)
{
DEFDBLOCVAR (catalog_mbx);
mqd_t s_mqd_t;
TOKEN mes[2];
UCHAR data[255];
char m[128];
struct mq_attr mqstat;
/*UCHAR priority;*/
struct  sched_param param;

  SchedGet(0,0,&param);
  param.sched_priority++;
  SchedSet(0,0,SCHED_FIFO ,&param);
	/* 
  priority=get_task_priority(0,0);  
  change_task_priority(0,0,priority+1);
	*/

  ps_catalog_mbx_ele = gps_catalog_mbx + (mailbox - OBJ_MBX);

  if ( ps_catalog_mbx_ele->id == mailbox )
    {
    /* open mbx */
 	  s_mqd_t = mq_open(ps_catalog_mbx_ele->name, 
	                  O_WRONLY ,
	                  0 ,
	                  NULL );
    if (s_mqd_t == -1)
      {
      sprintf(m,"QNXOBJ:<mbx_object_send_reply> Impossible open mbx >%s< (%d), job 0x%x",
				      ps_catalog_mbx_ele->name,
							mailbox,
							(unsigned int)get_job_id());
      CHK(m,errno);
      }

    /* send message to queue */
    mes[0]=object;
    mes[1]=reply;
    memcpy(data,mes,sizeof(mes));

    mq_getattr(s_mqd_t,
	               &mqstat);
    if (mqstat.mq_curmsgs>1000)
      {
      sprintf(m,"QNXOBJ:<mbx_object_send_reply> Impossible send msg mbx >%s<(%d) full, job 0x%x",
				      ps_catalog_mbx_ele->name,
							mailbox,
							(unsigned int)get_job_id());
      WRITE_TRACE(m);
      }
    else
      {
	  	mq_send(s_mqd_t,
	          data,
	          sizeof(mes),
	          0);
	    }
    mq_close(s_mqd_t);
    }
  /*    
  change_task_priority(0,0,priority);
	*/

  param.sched_priority--;
  SchedSet(0,0,SCHED_FIFO ,&param);


}
/*@..........................................................................
@  end  of mbx_object_send_reply
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       mbx_object_wait
@
@ DESCRIPTION   :       it waits on a object mailbox for time=time_limit
@
@ SYNOPSYS      :       status=mbx_object_wait(mailbox,objptr,time_limit);
@
@ PARAMETERS    :       TOKEN   mailbox = object mailbox token
@                       TOKEN   *objptr = ptr to object token received from
@                                         specified mailbox
@                       USHORT  time_limit = time to wait
@
@ RETURN        :       USHORT status:  0 = OK
@                       !=0 (E_TIME) = timeout
@
@ NOTE          :       for QNX use mbx_object_wait
@
@***************************************************************************/
USHORT mbx_object_wait(TOKEN mailbox,TOKEN *objptr,USHORT time_limit)
{
TOKEN far *reply_ptr;
  
  return(mbx_object_wait_reply(mailbox,objptr,time_limit,reply_ptr));

}
/*@..........................................................................
@  end  of mbx_object_wait
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       mbx_object_wait_forever
@
@ DESCRIPTION   :       it waits forever on a object mailbox
@
@ SYNOPSYS      :       mbx_object_wait_forever(mailbox,objptr);
@
@ PARAMETERS    :       TOKEN   mailbox = object mailbox token
@                       TOKEN   *objptr = ptr to object token received from
@                                         specified mailbox
@
@ RETURN        :       none
@
@ NOTE          :       Abort task in case of error (CHK macro)
@
@***************************************************************************/
void mbx_object_wait_forever(TOKEN mailbox,TOKEN *objptr)
{
	TOKEN  reply;

	mbx_object_wait_reply(mailbox,objptr,WAIT_FOREVER,&reply);
}
/*@..........................................................................
@  end  of mbx_object_wait_forever
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       mbx_object_wait_reply
@
@ DESCRIPTION   :       it waits on a object mailbox for time=time_limit
@                       and returns the TOKEN of the semaphore or mailbox
@                       where to send a reply
@
@ SYNOPSYS      :       status=mbx_object_wait_reply(mailbox,objptr,time_limit,reply_ptr);
@
@ PARAMETERS    :       TOKEN   mailbox = object mailbox token
@                       TOKEN   *objptr = ptr to object token received from
@                                         specified mailbox
@                       USHORT  time_limit = time to wait
@                       TOKEN far *reply_ptr=object token where to send a reply
@
@ RETURN        :       USHORT status:  0 = OK
@                                     !=0 (E_TIME) = timeout
@
@ NOTE          :       Abort task in case of error (CHK macro)
@
@***************************************************************************/
USHORT mbx_object_wait_reply(TOKEN mailbox,TOKEN *objptr,USHORT time_limit,TOKEN far *reply_ptr)
{
USHORT ldata;
TOKEN mes[2];
UCHAR data[255];

  if (time_limit!=WAIT_FOREVER)
    {
    ldata=mbx_data_wait(mailbox,data,time_limit*TICK);
    }
  else
    {
    ldata=mbx_data_wait_forever(mailbox,data);
    }

  if (ldata==(sizeof(TOKEN)*2))
    {
    memcpy(mes,data,sizeof(mes));
    memcpy(objptr,mes,sizeof(TOKEN));
    memcpy(reply_ptr,&mes[1],sizeof(TOKEN));
    return(E_OK);
    }
  else
    {
    return(!E_OK);
    } 

}
/*@..........................................................................
@  end  of mbx_object_wait_reply
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       mbx_object_wait_reply_forever
@
@ DESCRIPTION   :       it waits forever on a object mailbox
@                       and return the TOKEN of the semaphore or mailbox
@                       where to send a reply
@
@ SYNOPSYS      :       mbx_object_wait_reply_forever(mailbox,objptr,reply_ptr);
@
@ PARAMETERS    :       TOKEN   mailbox = object mailbox token
@                       TOKEN   *objptr = ptr to object token received from
@                                         specified mailbox
@                       TOKEN far *reply_ptr=object token where to send a reply
@
@ RETURN        :       none
@
@ NOTE          :       Abort task in case of error (CHK macro)
@
@***************************************************************************/
void mbx_object_wait_reply_forever(TOKEN mailbox,TOKEN *objptr,TOKEN far *reply_ptr)
{

	mbx_object_wait_reply(mailbox,objptr,WAIT_FOREVER,reply_ptr);

}
/*@..........................................................................
@  end  of mbx_object_wait_reply_forever
@***************************************************************************/
/*@**************************************************************************
@
@ FUNCTION      :       create_io_job
@
@ DESCRIPTION   :       it creates an I/O job
@
@ SYNOPSYS      :       mbx=create_io_job(job_path,job_name,priority);
@
@ PARAMETERS    :       char *job_path = job path
@                       char *job_name = job name
@                       UCHAR priority = job priority(=first task of the job)
@
@ RETURN        :       TOKEN mbx=mailbox where job send exit message
@
@ NOTE          :       Abort job/task in case of error (CHK macro)
@
@***************************************************************************/
TOKEN create_io_job(char *job_path,char *job_name,UCHAR priority)
{
struct 	stat statbuf;
char    program[80],
				program_original[80],
				message[128],
				next[20];
/*TOKEN   msg_d_mbx;*/
KN_TOKEN io_job;
int  status;
FILE    *fp;
POINTER p;
int i;

	if (job_path[0]!='/')
    {
    i=0;
	  FOREVER
		  {
      switch (i)
        {
        case 0:
          p=(POINTER)getenv("xbuser");
          if (p)
		        strcpy(next, p);
          break;

        case 1:
          p=(POINTER)getenv("xbkernel");
          if (p)
		        strcpy(next, p);
          break;

        case 2:
          strcpy(next,"/xb");
          break;

        default:
			    sprintf(message,"fopen error: I/O job %s file not found\n",program);
			    status=E_EXIST;
			    CHK(message,status);
        }
		  sprintf(program,"%s/%s/%s.qnx",
                      next,
                      job_path,
                      job_name);

      strlwr(program);
		  /* it tests if the file exists: driver or program exist */
		  if ((fp=fopen(program,"rb")) != 0)
        {
    	  fclose(fp);
        break;
        }
      i+=1;
      }
    }
  else
    {
		sprintf(program,"%s/%s.qnx",job_path,job_name);
    strlwr(program);
		/* it tests if the file exists: driver or program exist */
		if ((fp=fopen(program,"rb")) != 0)
      {
    	fclose(fp);
      }
    else
      {
			sprintf(message,"fopen error: I/O job %s file not found\n",program);
			status=E_EXIST;
			CHK(message,status);
      }
    }

	strcpy(program_original,program);
	io_job = spawnl(P_NOWAIT,program,program,NULL);
  
	if (io_job == 0)
		{
		sprintf(message,"create_io_job failed : %s not loaded \n",program_original);
    WRITE_TRACE(message);
		}
	else
		{
		sprintf(message,"create_io_job: %s job loaded (father job=0x%lx",program_original,get_job_id());
		WRITE_TRACE(message);
		}

	catalog_job_local(job_name,io_job);

  /* set priority */
  change_task_priority(0,io_job,priority);

	return(E_OK);/* return E_OK */

}
/*@..........................................................................
@  end of create_io_job
@***************************************************************************/


/*@**************************************************************************
@
@ FUNCTION      :       sleep_task
@
@ DESCRIPTION   :       it puts the task in wait for xx ms
@
@ SYNOPSYS      :       sleep_time(time_to_wait);
@
@ PARAMETERS    :       ULONG time_to_wait=time to wait in TICK
@
@ RETURN        :       none
@
@ NOTE          :       
@
@***************************************************************************/
void sleep_task(ULONG tick_to_wait)
{

	/* get seconds or the ms to sleep (because usleep can sleep for 1 second maximum) */
	if (tick_to_wait>100) /* one TICK=10ms */
		{
		/* get the seconds to sleep  and sleep for the requested time */
		sleep (tick_to_wait/100); /* sleep for the requested seconds */
		/* get the reaming tick to wait (less than 1 second)*/
		tick_to_wait=tick_to_wait%100;
		}
	if (tick_to_wait>0)
		{
		/* get the sleep time in microseconds and sleep for the requested time */
		usleep(tick_to_wait * 10000L); /* sleep for the requested microseconds */
		}
}
/*@..........................................................................
@  end  of sleep_task
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       change_priority
@
@ DESCRIPTION   :       it changes the task priority
@
@ SYNOPSYS      :       change_priority(new_priority);
@
@ PARAMETERS    :       UCHAR new_priority=new task priority
@
@ RETURN        :       none
@
@ NOTE          :       
@
@***************************************************************************/
void change_priority(UCHAR new_priority)
{

	change_task_priority(0,0,new_priority);

}
/*@..........................................................................
@  end  of change_priority
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       get_priority
@
@ DESCRIPTION   :       it gets the task priority (self task)
@
@ SYNOPSYS      :       priority=get_priority();
@
@ PARAMETERS    :       none
@
@ RETURN        :       UCHAR priority=task priority
@
@ NOTE          :       return the same priority value used by iRMX
@
@***************************************************************************/
UCHAR get_priority(void)
{
UCHAR priority;
  
  priority=get_task_priority(0,0);

  if(priority>=64)
    return (MAX_PRIORITY);
  else 
    {
    if(priority==0)
      return (254);
    else if(priority<15)
      return (160);
    }

	return(160+15-priority);

}
/*@..........................................................................
@  end  of get_priority
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       get_task_priority
@
@ DESCRIPTION   :       it gets the task priority (any task)
@
@ SYNOPSYS      :       priority=get_task_priority(task,job);
@
@ PARAMETERS    :       TOKEN task=taks token
@												KN_TOKEN job= job id
@
@ RETURN        :       UCHAR priority=task priority
@
@ NOTE          :       
@
@***************************************************************************/
UCHAR get_task_priority(TOKEN task,KN_TOKEN job)
{
	struct  sched_param param;
	int res;
  
  res = SchedGet(job,task,&param);
  if (res!=-1)
    {
    /*return(param.sched_curpriority);*/
    return(param.sched_priority);
    }
  else
    {
  	return(0);
    }

}
/*@..........................................................................
@  end  of get_task_priority
@***************************************************************************/


/*@**************************************************************************
@
@ FUNCTION      :       change_task_priority
@
@ DESCRIPTION   :       it changes the task priority (any task)
@
@ SYNOPSYS      :       change_task_priority(task,job,new_priority);
@
@ PARAMETERS    :       KN_TOKEN task=taks token
@                       KN_TOKEN job = job id
@                       UCHAR new_priority=new task priority
@
@ RETURN        :       none
@
@ NOTE          :       in case of error the priority is not changed
@
@***************************************************************************/
void change_task_priority(TOKEN task,KN_TOKEN job,UCHAR new_priority)
{
	struct  sched_param param;
	int res;
  char m[128];
#ifdef TEST_OS_LIB
  KN_TOKEN io_job;
#endif

  /* get the current attributes */
  res = SchedGet(job,task,&param);
  if (res==-1)
    {
    sprintf(m,"QNXOBJ:<change_task_priority> Impossible change priority for thread=0x%8x job=0x%8x",
              (unsigned int)task,
              (unsigned int)job);
    WRITE_TRACE(m);
    }

  param.sched_priority=GetQnxPriority(new_priority);

#ifdef TEST_OS_LIB
    io_job = get_job_id;
    sprintf(m,"QNXOBJ:<change_task_priority> Change to priority %d (%d) for thread=0x%8x job=0x%8x",
              new_priority,
              param.sched_priority,
              (unsigned int)task,
              io_job);
    WRITE_TRACE(m);
#endif

  res = SchedSet(job,task,SCHED_FIFO ,&param);
  if (res==-1)
    {
    sprintf(m,"QNXOBJ:<change_task_priority> Impossible change priority for thread=0x%8x job=0x%8x",
              (unsigned int)task,
              (unsigned int)job);
    WRITE_TRACE(m);
    }

}
/*@..........................................................................
@  end  of change_task_priority
@***************************************************************************/


/*@**************************************************************************
@
@ FUNCTION      :       is_valid_mbx
@
@ DESCRIPTION   :       test if the object is a mailbox
@
@ SYNOPSYS      :       res=is_valid_mbx(mbx);
@
@ PARAMETERS    :       TOKEN mbx = mailbox token
@
@ RETURN        :       USHORT res
@                       (0=no mbx, 1= OK mbx)
@
@ NOTE          :       None
@
@***************************************************************************/
USHORT is_valid_mbx(TOKEN mbx)
{

  if (get_object_type(mbx)!=T_MAILBOX)
    return 0;
  else
    return 1;

}
/*@..........................................................................
@  end  of is_valid_mbx
@***************************************************************************/


/*@**************************************************************************
@
@ FUNCTION      :       get_object_type
@
@ DESCRIPTION   :       it gets the type of the object
@
@ SYNOPSYS      :       type=get_object_type(object);
@
@ PARAMETERS    :       TOKEN object=object token
@
@ RETURN        :       USHORT type=object type
@                       (0=object inexistent)
@
@ NOTE          :       None
@
@***************************************************************************/
USHORT get_object_type(TOKEN object)
{
  DEFDBLOCVAR (catalog_mbx);
  DEFDBLOCVAR (catalog_sem);
  DEFDBLOCVAR (catalog_seg);
  DEFDBLOCVAR (catalog_reg);
  DEFDBLOCVAR (catalog_tsk);

	/* If object=0: not valid
	   Scan object table in reverse order depending by the OBJ_xxx define (see xbtypes.h) */

	if (!object)
		return(0);

	if (object>=OBJ_SEG)
		{
		SCANDB(catalog_seg)
			{
			if (!ps_catalog_seg_ele->id)
				return(0);
			if ( ps_catalog_seg_ele->id == object )
				return(T_SEGMENT);
			}
		}
	else 	if (object>=OBJ_TSK)
		{
		SCANDB(catalog_tsk)
			{
			if (!ps_catalog_tsk_ele->id)
				return(0);
			if ( ps_catalog_tsk_ele->id == object )
				return(T_TASK);
			}
		}
	else 	if (object>=OBJ_SEM)
		{
		SCANDB(catalog_sem)
			{
			if (!ps_catalog_sem_ele->id)
				return(0);
			if ( ps_catalog_sem_ele->id == object )
				return(T_SEMAPHORE);
			}
		}
	else 	if (object>=OBJ_MBX)
		{
		SCANDB(catalog_mbx)
			{
			if (!ps_catalog_mbx_ele->id)
				return(0);
			if ( ps_catalog_mbx_ele->id == object )
				return(T_MAILBOX);
			}
		}
	else
		{
		SCANDB(catalog_reg)
			{
			if (!ps_catalog_reg_ele->id)
				return(0);
			if ( ps_catalog_reg_ele->id == object )
				return(T_REGION);
			}
		}

  return(0);
}
/*@..........................................................................
@  end  of get_object_type
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       GetQnxPriority
@
@ DESCRIPTION   :       return the QNX priority from iRMX/QNX priority
@
@ SYNOPSYS      :       QNXpriority=GetQnxPriority(priority);
@
@ PARAMETERS    :       UCHAR priority
@
@ RETURN        :       UCHAR = QNX priority
@
@ NOTE          :       None
@
@***************************************************************************/
UCHAR GetQnxPriority(UCHAR priority)
{

  if (priority<MAX_PRIORITY)
    return (priority); /* this is  a QNX priority*/

  if(priority==MIN_PRIORITY)
    return (1);
  else if(priority>(RR_PRIORITY-10))
    return (10);

	return(160+15-priority);

}
/*@..........................................................................
@  end  of GetQnxPriority
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       create_alarm_task
@
@ DESCRIPTION   :       create an alarm thread
@
@ SYNOPSYS      :       res=create_alarm_task(object);
@
@ PARAMETERS    :       TOKEN object=object token
@
@ RETURN        :       USHORT type=object type
@                       (0=object inexistent)
@
@ NOTE          :       The AlarmTask must be created at the highest 
@                       QNX priority (=63)
@
@***************************************************************************/
TOKEN create_alarm_task(ULONG *KN_area ,void *function,ULONG time_limit,KN_FLAGS flags)
{
  alarm_function=function;
  alarm_time_limit=time_limit;
  alarm_flags=flags;
	create_normal_task("TSK_ALARM",AlarmTask,HIGH_PRIORITY-2);
  return (1); 
}
/*@..........................................................................
@  end  of create_alarm_task 
@***************************************************************************/

/*@**************************************************************************
@
@ THREAD        :       AlarmTask 
@
@ DESCRIPTION   :       call the alarm function every time_limit msec
@
@ SYNOPSYS      :       void AlarmTask (void);
@
@ PARAMETERS    :       in the static variables :
@                           alarm_function
@                           alarm_time_limit
@                           alarm_flags
@
@ RETURN        :       none
@
@ NOTE          :       None
@
@***************************************************************************/
void AlarmTask (void)
{
void (*function)();
ULONG time_limit_nsec;
ULONG time_limit_sec;
KN_FLAGS flags;

int                     chid;
int                     rcvid;
timer_t                 timer_id;
struct sigevent         event;
struct itimerspec       itime;
struct _pulse   pulse;
struct  sched_param param;

#ifdef DEBUG_ALARM_TASK
struct timespec start_time, end_time, f_time;
#endif

  /* It is necessary set the priority here in order to have it in this alarm task */
  SchedGet(0,0,&param);
  param.sched_priority=62;
  SchedSet(0,0,SCHED_FIFO ,&param);

  /* initialization */
  function=alarm_function;
  time_limit_sec=alarm_time_limit/1000000000;
  time_limit_nsec=(alarm_time_limit-time_limit_sec*1000)*1000000; /* convert alarm_time_limit in nanoseconds */
  flags=alarm_flags;

  chid = ChannelCreate(0);

  event.sigev_notify = SIGEV_PULSE;
  event.sigev_coid = ConnectAttach(NULL,  /* local node */
                                   0, 
                                   chid, 
                                   _NTO_SIDE_CHANNEL, 
                                   0);

  event.sigev_priority = getprio(0);

  event.sigev_code = ALARM_TASK_PULSE_CODE;

  timer_create(CLOCK_REALTIME, &event, &timer_id);

  itime.it_value.tv_sec = time_limit_sec;
  itime.it_value.tv_nsec = time_limit_nsec;
  itime.it_interval.tv_sec = time_limit_sec;
  itime.it_interval.tv_nsec = time_limit_nsec;

  timer_settime(timer_id, 0, &itime, NULL);


  FOREVER
    {
    /* every time limit microseconds call function */

#ifdef DEBUG_ALARM_TASK
    clock_gettime( CLOCK_REALTIME, &start_time); 
#endif

    rcvid = MsgReceive(chid, &pulse, sizeof(pulse), NULL);
    if (rcvid == 0) 
      { /* we got a pulse */
      if (pulse.code == ALARM_TASK_PULSE_CODE)
        {
#ifdef DEBUG_ALARM_TASK
        clock_gettime( CLOCK_REALTIME, &f_time); 
#endif

        function();

#ifdef DEBUG_ALARM_TASK
        clock_gettime( CLOCK_REALTIME, &end_time);
#endif
        }
      }
    }
}
/*@..........................................................................
@  end  of AlarmTask task
@***************************************************************************/



/*@**************************************************************************
@
@ FUNCTION      :       attach_catalogs
@
@ DESCRIPTION   :       map the catalog tables
@
@ SYNOPSYS      :       attach_catalogs;
@
@ PARAMETERS    :       
@
@ RETURN        :       
@
@ NOTE          :       None
@
@***************************************************************************/
void attach_catalogs(void)
{
ULONG len;
POINTER p_sh;
int bNew=0;
int fd;

  gn_catalog_sem = MAX_CATALOG_ITEM ;
  gn_catalog_reg = MAX_CATALOG_ITEM ;
  gn_catalog_mbx = MAX_CATALOG_ITEM ;
  gn_catalog_tsk = MAX_CATALOG_ITEM ;
  gn_catalog_seg = MAX_CATALOG_ITEM * 5;
  gn_catalog_gen = MAX_CATALOG_GEN ;

  gl_catalog_reg = sizeof(S_catalog_reg);
  gl_catalog_sem = sizeof(S_catalog_sem);
  gl_catalog_mbx = sizeof(S_catalog_mbx);
  gl_catalog_tsk = sizeof(S_catalog_tsk);
  gl_catalog_gen = sizeof(S_catalog_gen);
  gl_catalog_seg = sizeof(S_catalog_seg);
  
  /* Map the memory object */
  len= 4096 +
       gl_catalog_seg * gn_catalog_seg +
       gl_catalog_sem * gn_catalog_sem +
       gl_catalog_reg * gn_catalog_reg +
       gl_catalog_mbx * gn_catalog_mbx +
       gl_catalog_tsk * gn_catalog_tsk +
       gl_catalog_gen * gn_catalog_gen ;
	
	{
	CHAR name[16];
	sprintf(name,"/CATALOGS");
	p_sh =map_sh_mem(name,len);
	}

  gps_catalog_ini=p_sh;	
  p_sh += 4096;
  
  gps_catalog_reg = (S_catalog_reg*) p_sh;
  p_sh += gl_catalog_reg * gn_catalog_reg;
  
  gps_catalog_sem = (S_catalog_sem*) p_sh;
  p_sh += gl_catalog_sem * gn_catalog_sem; 
  
  gps_catalog_mbx = (S_catalog_mbx*) p_sh;
  p_sh += gl_catalog_mbx * gn_catalog_mbx;
  
  gps_catalog_tsk = (S_catalog_tsk*) p_sh;
  p_sh += gl_catalog_tsk * gn_catalog_tsk;
  
  gps_catalog_gen = (S_catalog_gen*) p_sh;
  p_sh += gl_catalog_gen * gn_catalog_gen;

  gps_catalog_seg = (S_catalog_seg*) p_sh;
  p_sh += gl_catalog_seg * gn_catalog_seg;  
  
}
/*@..........................................................................
@  end  of attach_catalogs
@***************************************************************************/


/*@**************************************************************************
@
@ FUNCTION      :       attach_task_sched
@
@ DESCRIPTION   :       map the task_sched tables
@
@ SYNOPSYS      :       attach_task_sched;
@
@ PARAMETERS    :       
@
@ RETURN        :       
@
@ NOTE          :       None
@
@***************************************************************************/
void attach_task_sched(void)
{
ULONG len;
POINTER p_sh;
int bNew=0;
int fd;

  gn_task_sched = (USHORT) TASK_SCHED_TB;

  gl_task_sched = sizeof(S_task_sched);
  
  /* Map the memory object */
  len= gl_task_sched * gn_task_sched ;

	{
	CHAR name[16];
	sprintf(name,"/TASK_SCHED");
	p_sh =map_sh_mem(name,len);
	}

	gps_task_sched = (S_task_sched *) p_sh;

}
/*@..........................................................................
@  end  of attach_task_sched 
@***************************************************************************/


/*@**************************************************************************
@
@ FUNCTION      :       filelength
@
@ DESCRIPTION   :       return the file lenght
@
@ SYNOPSYS      :       flen=filelength(fdes);
@
@ PARAMETERS    :       int fdes : file descriptor
@
@ RETURN        :       long flen : file len
@
@ NOTE          :       None
@
@***************************************************************************/
long filelength(int fdes)
{

struct stat buf;
int rc;
	
	rc = fstat( fdes , &buf );
	
	return (buf.st_size);
}
/*@..........................................................................
@  end  of filelength
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       is_valid_task
@
@ DESCRIPTION   :       checks if the task is valid
@
@ SYNOPSYS      :       res=is_valid_task(task,job);
@
@ PARAMETERS    :       TOKEN task=taks token
@												KN_TOKEN job= job id
@
@ RETURN        :       USHORT res (0= not OK)
@
@ NOTE          :       
@
@***************************************************************************/
USHORT is_valid_task(TOKEN task,KN_TOKEN job)
{
	struct  sched_param param;
	int res;
  
  res = SchedGet(job,task,&param);
  if (res==-1)
    {
    return(0); /* there is an error */
    }
  else
    {
  	return(1);
    }

}
/*@..........................................................................
@  end of is_valid_task
@***************************************************************************/

/*@**************************************************************************
@
@ FUNCTION      :       is_valid_job
@
@ DESCRIPTION   :       checks if the job is valid
@
@ SYNOPSYS      :       res=is_valid_job(job);
@
@ PARAMETERS    :       KN_TOKEN job= job id
@
@ RETURN        :       USHORT res (0= not OK)
@
@ NOTE          :       
@
@***************************************************************************/
USHORT is_valid_job(KN_TOKEN job)
{
	struct  sched_param param;
	int res;
  
  res = SchedGet(job,0,&param);
  if (res==-1)
    {
    return(0); /* there is an error */
    }
  else
    {
  	return(1);
    }

}
/*@..........................................................................
@  end of is_valid_job
@***************************************************************************/
#endif
